@@ -186,7 +186,6 @@ struct Window::PrivateData {
         puglInitContextType(fView, PUGL_GL);
         puglInitUserResizable(fView, fResizable);
         puglInitWindowSize(fView, static_cast<int>(fWidth), static_cast<int>(fHeight));
-        puglIgnoreKeyRepeat(fView, true);
 
         puglSetHandle(fView, this);
         puglSetDisplayFunc(fView, onDisplayCallback);
@@ -911,7 +910,6 @@ struct Window::PrivateData {
 
     static int onSpecialCallback(PuglView* view, bool press, PuglKey key)
     {
-        printf("special pugl key...\n");
         return handlePtr->onPuglSpecial(press, static_cast<Key>(key));
     }
 
