@@ -289,6 +289,14 @@ struct Parameter {
     */
     ParameterRanges ranges;
 
+   /**
+      MIDI CC to use by default on this parameter.@n
+      A value of 0 or 32 (bank change) is considered invalid.@n
+      Must also be less or equal to 120.
+      @note This value is only a hint! Hosts might map it automatically or completely ignore it.
+    */
+    uint8_t midiCC;
+
    /**
       Default constructor for a null parameter.
     */
@@ -297,7 +305,8 @@ struct Parameter {
           name(),
           symbol(),
           unit(),
-          ranges() {}
+          ranges(),
+          midiCC(0) {}
 
    /**
       Constructor using custom values.
@@ -307,7 +316,8 @@ struct Parameter {
           name(n),
           symbol(s),
           unit(u),
-          ranges(def, min, max) {}
+          ranges(def, min, max),
+          midiCC(0) {}
 };
 
 /**
@@ -529,6 +539,9 @@ public:
       Write a MIDI output event.@n
       This function must only be called during run().@n
       Returns false when the host buffer is full, in which case do not call this again until the next run().
+      @note This function is not implemented yet!@n
+            It's here so that developers can prepare MIDI plugins in advance.@n
+            If you plan to use this, please report to DPF authos so it can be implemented.
     */
     bool writeMidiEvent(const MidiEvent& midiEvent) noexcept;
 #endif
