@@ -35,12 +35,9 @@
 #include <functional>
 #include <initializer_list>
 #include <rtosc/rtosc.h>
-#include <rtosc/rtosc-version.h>
-#include <cstring>
-#include <cctype>
-#include <cstdlib>
-#include <cstdio>
 #include <string>
+#include <cstdio>
+#include <iosfwd>
 
 namespace rtosc {
 
@@ -259,165 +256,6 @@ struct MergePorts:public Ports
     MergePorts(std::initializer_list<const Ports*> c);
 };
 
-/**
- * Return a port's default value
- *
- * Returns the default value of a given port, if any exists, as a string.
- * For the parameters, see the overloaded function.
- * @note The recursive parameter should never be specified.
- * @return The default value(s), pretty-printed, or NULL if there is no
- *     valid default annotation
- */
-const char* get_default_value(const char* port_name, const Ports& ports,
-                              void* runtime, const Port* port_hint = NULL,
-                              int32_t idx = -1, int recursive = 1);
-
-/**
- * Return a port's default value
- *
- * Returns the default value of a given port, if any exists, as an array of
- * rtosc_arg_vals . The values in the resulting array are being canonicalized,
- * i.e. mapped values are being converted to integers; see
- * canonicalize_arg_vals() .
- *
- * @param port_name the port's OSC path.
- * @param port_args the port's arguments, e.g. '::i:c:S'
- * @param ports the ports where @a portname is to be searched
- * @param runtime object holding @a ports . Optional. Helps finding
- *        default values dependent on others, such as presets.
- * @param port_hint The port itself corresponding to portname (including
- *        the args). If not specified, will be found using @p portname .
- * @param idx If the port is an array (using the '#' notation), this specifies
- *        the index required for the default value
- * @param n Size of the output parameter @res . This size can not be known,
- *        so you should provide a large enough array.
- * @param res The output parameter for the argument values.
- * @param strbuf String buffer for storing pretty printed strings and blobs.
- * @param strbufsize Size of @p strbuf
- * @return The actual number of aruments written to @p res (can be smaller
- *         than @p n) or -1 if there is no valid default annotation
- */
-int get_default_value(const char* port_name, const char *port_args,
-                      const Ports& ports,
-                      void* runtime, const Port* port_hint,
-                      int32_t idx,
-                      size_t n, rtosc_arg_val_t* res,
-                      char *strbuf, size_t strbufsize);
-
-
-/**
- * Return a string list of all changed values
- *
- * Return a human readable list of the value that changed
- * corresponding to the rDefault macro
- * @param ports The static ports structure
- * @param runtime The runtime object
- * @note This function is not realtime save (It uses std::string), which is
- *   usually OK, since this function is being run inside a non-RT thread. If you
- *   need this to be realtime save, add a template parameter for a functor that
- *   replaces the std::string handling.
- * @return The list of ports and their changed values, linewise
- */
-std::string get_changed_values(const Ports& ports, void* runtime);
-
-//! @brief Class to modify and dispatch messages loaded from savefiles.
-//! Objects of this class shall be passed to savefile loading routines. You can
-//! inherit to change the behaviour, e.g. to modify or discard such messages.
-class savefile_dispatcher_t
-{
-    const Ports* ports;
-    void* runtime;
-    char loc[1024];
-
-protected:
-    enum proceed {
-        abort = -2, //!< the message shall lead to abort the savefile loading
-        discard = -1 //!< the message shall not be dispatched
-    };
-
-    enum dependency_t {
-        no_dependencies,  //! default values don't depend on others
-        has_dependencies, //! default values do depend on others
-        not_specified     //! it's not know which of the other enum values fit
-    };
-
-    rtosc_version rtosc_filever, //!< rtosc versinon savefile was written with
-                  rtosc_curver, //!< rtosc version of this library
-                  app_filever, //!< app version savefile was written with
-                  app_curver; //!< current app version
-
-    //! call this to dispatch a message
-    bool operator()(const char* msg) { return do_dispatch(msg); }
-
-    static int default_response(size_t nargs, bool first_round,
-                                dependency_t dependency);
-
-private:
-    //! callback for when a message shall be dispatched
-    //! implement this if you need to change a message
-    virtual int on_dispatch(size_t portname_max, char* portname,
-                            size_t maxargs, size_t nargs,
-                            rtosc_arg_val_t* args,
-                            bool round2, dependency_t dependency);
-    //! call this to dispatch a message
-    virtual bool do_dispatch(const char* msg);
-
-    friend int dispatch_printed_messages(const char* messages,
-                                         const Ports& ports, void* runtime,
-                                         savefile_dispatcher_t *dispatcher);
-
-    friend int load_from_file(const char* file_content,
-                              const Ports& ports, void* runtime,
-                              const char* appname,
-                              rtosc_version appver,
-                              savefile_dispatcher_t* dispatcher);
-};
-
-/**
- * Scan OSC messages from human readable format and dispatch them.
- * @param messages The OSC messages, whitespace-separated
- * @param ports The static ports structure
- * @param runtime The runtime object
- * @param dispatcher Object to modify messages prior to dispatching, or NULL.
- *   You can overwrite its virtual functions, and you should specify any of the
- *   version structs if needed. All other members shall not be initialized.
- * @return The number of messages read, or, if there was a read error,
- *   or the dispatcher did refuse to dispatch,
- *   the number of bytes read until the read error occured minus one
- */
-int dispatch_printed_messages(const char* messages,
-                              const Ports& ports, void* runtime,
-                              savefile_dispatcher_t *dispatcher = NULL);
-
-/**
- * Return a savefile containing all values that differ from the default values.
- * @param ports The static ports structure
- * @param runtime The runtime object
- * @param appname Name of the application calling this function
- * @param appver Version of the application calling this function
- * @return The resulting savefile as an std::sting
- */
-std::string save_to_file(const Ports& ports, void* runtime,
-                         const char* appname, rtosc_version appver);
-
-/**
- * Read save file and dispatch contained parameters.
- * @param file_content The file as a C string
- * @param ports The static ports structure
- * @param runtime The runtime object
- * @param appname Name of the application calling this function; must
- *   match the file's application name
- * @param appver Version of the application calling this function
- * @param dispatcher Modifier for the messages; NULL if no modifiers are needed
- * @return The number of messages read, or, if there was a read error,
- *   the negated number of bytes read until the read error occured minus one
- */
-int load_from_file(const char* file_content,
-                   const Ports& ports, void* runtime,
-                   const char* appname,
-                   rtosc_version appver,
-                   savefile_dispatcher_t* dispatcher = NULL);
-
 /**
  * Convert given argument values to their canonical representation.
  *
@@ -487,6 +325,51 @@ void walk_ports(const Ports *base,
                 bool expand_bundles = true,
                 void *runtime = NULL);
 
+/**
+ * Returns paths and metadata of all direct children of a port, or of the port
+ * itself if that port has no children.
+ * If you just want to generate a reply message, use the overloaded function.
+ * @param root see @p m
+ * @param str location under @p root to look up port, or empty-string to search
+ *            directly at @p root
+ * @param needle Only port names starting with this sting are returned (use
+ *            empty-string or nullptr to match everything)
+ * @param types A buffer where the OSC type string is being written
+ * @param max_types should be @p max_args +1 for best performance,
+ *   see @p max_args
+ * @param args An array where the argument values are being written
+ * @param max_args maximum number of arguments in array at @p args. Should be
+ *   greater or equal than 2 * (maximum no. of child ports of your app's ports).
+ */
+void path_search(const rtosc::Ports& root, const char *str, const char *needle,
+                 char *types, std::size_t max_types,
+                 rtosc_arg_t* args, std::size_t max_args);
+
+/**
+ * Returns a messages of all paths and metadata of all direct children of a
+ * port, or of the port itself if that port has no children.
+ *
+ * Your app should always have a port "path-search", calling this function
+ * directly, and replying the resulting buffer @p msgbuf directly. That way, it
+ * will be ready for oscprompt, port-checker etc.
+ *
+ * @param root See @p m
+ * @param m a valid OSC message requesting the path search. The corresponding
+ *   port args must be of types
+ *   * "s" (location under @p root to look up port, or empty-string to search
+ *          directly at @p root)
+ *   * "s" (only port names starting with this sting are returned, use
+ *          empty-string or nullptr to match everything)
+ * @param max_ports Maximum number (or higher) of child ports in any of your
+ *     app's ports.
+ * @param msgbuf Buffer for the reply message
+ * @param bufsize Size of the message buffer @p msgbuf
+ * @return The length of the reply message (0 means error)
+ */
+std::size_t path_search(const rtosc::Ports& root, const char *m,
+                        std::size_t max_ports,
+                        char *msgbuf, std::size_t bufsize);
+
 /**
  * Return the index with value @p value from the metadata's enumeration.
  * @param meta The metadata
