@@ -25,11 +25,13 @@
 /**
  * @file rtosc-version.h
  * Definition of rtosc's version struct
+ *
  * @note the implementation is in version.c.in
+ * @test version.c
  */
 
-#ifndef RTOSC_VERSION_H
-#define RTOSC_VERSION_H
+#ifndef RTOSC_VERSION
+#define RTOSC_VERSION
 
 #ifdef __cplusplus
 extern "C" {
@@ -64,5 +66,5 @@ void rtosc_version_print_to_12byte_str(const rtosc_version* v,
 };
 #endif
 
-#endif
+#endif // RTOSC_VERSION
 
