@@ -8,6 +8,7 @@
 
 #include <rtosc/rtosc.h>
 #include <rtosc/arg-val-math.h>
+#include "util.h"
 
 const char *rtosc_argument_string(const char *msg)
 {
@@ -233,20 +234,14 @@ static size_t vsosc_null(const char        *address,
     return pos;
 }
 
-static const rtosc_cmp_options default_cmp_options = { 0.0 };
-
-void rtosc_arg_val_itr_init(rtosc_arg_val_t_const_itr* itr,
+void rtosc_arg_val_itr_init(rtosc_arg_val_itr* itr,
                             const rtosc_arg_val_t* av)
 {
     itr->av = av;
     itr->i = itr->range_i = 0;
 }
 
-//! helper function for arg val comparing
-//! this usually just returns the value from operand, except for range operands,
-//! where the value is being interpolated
-const rtosc_arg_val_t* rtosc_arg_val_itr_get(
-    const rtosc_arg_val_t_const_itr* itr,
+const rtosc_arg_val_t* rtosc_arg_val_itr_get(const rtosc_arg_val_itr *itr,
     rtosc_arg_val_t* buffer)
 {
     const rtosc_arg_val_t* result;
@@ -262,7 +257,7 @@ const rtosc_arg_val_t* rtosc_arg_val_itr_get(
     return result;
 }
 
-void rtosc_arg_val_itr_next(rtosc_arg_val_t_const_itr* itr)
+void rtosc_arg_val_itr_next(rtosc_arg_val_itr *itr)
 {
     // increase the range index
     if(itr->av->type == '-')
@@ -294,296 +289,6 @@ void rtosc_arg_val_itr_next(rtosc_arg_val_t_const_itr* itr)
     }
 }
 
-// abort only if one side was finished, or if both are infinite ranges
-int rtosc_arg_vals_cmp_has_next(const rtosc_arg_val_t_const_itr* litr,
-                                const rtosc_arg_val_t_const_itr* ritr,
-                                size_t lsize, size_t rsize)
-{
-    return     (litr->i < lsize) && (ritr->i < rsize)
-            && (litr->av->type != '-' || ritr->av->type != '-' ||
-                litr->av->val.r.num || ritr->av->val.r.num);
-}
-
-// arrays are equal by now, but is one longer?
-// each array must have either been completely passed, or it must
-// have reached an infinite range
-int rtosc_arg_vals_eq_after_abort(const rtosc_arg_val_t_const_itr* litr,
-                                  const rtosc_arg_val_t_const_itr* ritr,
-                                  size_t lsize, size_t rsize)
-{
-    return    (litr->i == lsize ||
-                  (litr->av->type == '-' && !litr->av->val.r.num))
-           && (ritr->i == rsize ||
-                  (ritr->av->type == '-' && !ritr->av->val.r.num));
-}
-
-// compare single elements, ranges excluded
-int rtosc_arg_vals_eq_single(const rtosc_arg_val_t* _lhs,
-                             const rtosc_arg_val_t* _rhs,
-                             const rtosc_cmp_options* opt)
-{
-#define mfabs(val) (((val) >= 0) ? (val) : -(val))
-    int rval;
-
-    if(!opt)
-        opt = &default_cmp_options;
-
-    if(_lhs->type == _rhs->type)
-    switch(_lhs->type)
-    {
-        case 'i':
-        case 'c':
-        case 'r':
-            rval = _lhs->val.i == _rhs->val.i;
-            break;
-        case 'I':
-        case 'T':
-        case 'F':
-        case 'N':
-            rval = 1;
-            break;
-        case 'f':
-            rval = (opt->float_tolerance == 0.0)
-                   ? _lhs->val.f == _rhs->val.f
-                   : mfabs(_lhs->val.f - _rhs->val.f) <=
-                   (float)opt->float_tolerance;
-            break;
-        case 'd':
-            rval = (opt->float_tolerance == 0.0)
-                   ? _lhs->val.d == _rhs->val.d
-                   : mfabs(_lhs->val.d - _rhs->val.d) <=
-                   opt->float_tolerance;
-            break;
-        case 'h':
-            rval = _lhs->val.h == _rhs->val.h;
-            break;
-        case 't':
-            rval = _lhs->val.t == _rhs->val.t;
-            break;
-        case 'm':
-            rval = 0 == memcmp(_lhs->val.m, _rhs->val.m, 4);
-            break;
-        case 's':
-        case 'S':
-            rval = (_lhs->val.s == NULL || _rhs->val.s == NULL)
-                 ? _lhs->val.s == _rhs->val.s
-                 : (0 == strcmp(_lhs->val.s, _rhs->val.s));
-            break;
-        case 'b':
-        {
-            int32_t lbs = _lhs->val.b.len,
-                    rbs = _rhs->val.b.len;
-            rval = lbs == rbs;
-            if(rval)
-                rval = 0 == memcmp(_lhs->val.b.data, _rhs->val.b.data, lbs);
-            break;
-        }
-        case 'a':
-        {
-            if(     _lhs->val.a.type != _rhs->val.a.type
-               && !(_lhs->val.a.type == 'T' && _rhs->val.a.type == 'F')
-               && !(_lhs->val.a.type == 'F' && _rhs->val.a.type == 'T'))
-                rval = 0;
-            else
-                rval = rtosc_arg_vals_eq(_lhs+1, _rhs+1,
-                                         _lhs->val.a.len, _rhs->val.a.len,
-                                         opt);
-            break;
-        }
-        case '-':
-            assert(false);
-            break;
-    }
-    else
-    {
-        rval = 0;
-    }
-    return rval;
-#undef mfabs
-}
-
-int rtosc_arg_vals_eq(const rtosc_arg_val_t* lhs, const rtosc_arg_val_t* rhs,
-                      size_t lsize, size_t rsize,
-                      const rtosc_cmp_options* opt)
-{
-    // used if the value of lhs or rhs is range-computed:
-    rtosc_arg_val_t rlhs, rrhs;
-
-    rtosc_arg_val_t_const_itr litr, ritr;
-    rtosc_arg_val_itr_init(&litr, lhs);
-    rtosc_arg_val_itr_init(&ritr, rhs);
-
-    int rval = 1;
-
-    if(!opt)
-        opt = &default_cmp_options;
-
-    for( ; rtosc_arg_vals_cmp_has_next(&litr, &ritr, lsize, rsize) && rval;
-        rtosc_arg_val_itr_next(&litr),
-        rtosc_arg_val_itr_next(&ritr))
-    {
-        rval = rtosc_arg_vals_eq_single(rtosc_arg_val_itr_get(&litr, &rlhs),
-                                        rtosc_arg_val_itr_get(&ritr, &rrhs),
-                                        opt);
-    }
-
-    return rval
-        ? rtosc_arg_vals_eq_after_abort(&litr, &ritr, lsize, rsize)
-        : rval;
-}
-
-// compare single elements, ranges excluded
-int rtosc_arg_vals_cmp_single(const rtosc_arg_val_t* _lhs,
-                              const rtosc_arg_val_t* _rhs,
-                              const rtosc_cmp_options* opt)
-{
-#define cmp_3way(val1,val2) (((val1) == (val2)) \
-                            ? 0 \
-                            : (((val1) > (val2)) ? 1 : -1))
-#define mfabs(val) (((val) >= 0) ? (val) : -(val))
-
-    int rval;
-
-    if(!opt)
-        opt = &default_cmp_options;
-
-    if(_lhs->type == _rhs->type)
-    switch(_lhs->type)
-    {
-        case 'i':
-        case 'c':
-        case 'r':
-            rval = cmp_3way(_lhs->val.i, _rhs->val.i);
-            break;
-        case 'I':
-        case 'T':
-        case 'F':
-        case 'N':
-            rval = 0;
-            break;
-        case 'f':
-            rval = (opt->float_tolerance == 0.0)
-                   ? cmp_3way(_lhs->val.f, _rhs->val.f)
-                   : (mfabs(_lhs->val.f - _rhs->val.f)
-                      <= (float)opt->float_tolerance)
-                      ? 0
-                      : ((_lhs->val.f > _rhs->val.f) ? 1 : -1);
-            break;
-        case 'd':
-            rval = (opt->float_tolerance == 0.0)
-                   ? cmp_3way(_lhs->val.d, _rhs->val.d)
-                   : (mfabs(_lhs->val.d - _rhs->val.d)
-                      <= opt->float_tolerance)
-                      ? 0
-                      : ((_lhs->val.d > _rhs->val.d) ? 1 : -1);
-            break;
-        case 'h':
-            rval = cmp_3way(_lhs->val.h, _rhs->val.h);
-            break;
-        case 't':
-            // immediately is considered lower than everything else
-            // this means if you send two events to a client,
-            // one being "immediately" and one being different, the
-            // immediately-event has the higher priority, event if the
-            // other one is in the past
-            rval = (_lhs->val.t == 1)
-                   ? (_rhs->val.t == 1)
-                     ? 0
-                     : -1 // _lhs has higher priority => _lhs < _rhs
-                   : (_rhs->val.t == 1)
-                     ? 1
-                     : cmp_3way(_lhs->val.t, _rhs->val.t);
-            break;
-        case 'm':
-            rval = memcmp(_lhs->val.m, _rhs->val.m, 4);
-            break;
-        case 's':
-        case 'S':
-            rval = (_lhs->val.s == NULL || _rhs->val.s == NULL)
-                 ? cmp_3way(_lhs->val.s, _rhs->val.s)
-                 : strcmp(_lhs->val.s, _rhs->val.s);
-            break;
-        case 'b':
-        {
-            int32_t lbs = _lhs->val.b.len,
-                    rbs = _rhs->val.b.len;
-            int32_t minlen = (lbs < rbs) ? lbs : rbs;
-            rval = memcmp(_lhs->val.b.data, _rhs->val.b.data, minlen);
-            if(lbs != rbs && !rval)
-            {
-                // both equal until here
-                // the string that ends here is lexicographically smaller
-                rval = (lbs > rbs)
-                       ? _lhs->val.b.data[minlen]
-                       : -_rhs->val.b.data[minlen];
-            }
-
-            break;
-        }
-        case 'a':
-        {
-            int32_t llen = _lhs->val.a.len, rlen = _rhs->val.a.len;
-            if(     _lhs->val.a.type != _rhs->val.a.type
-               && !(_lhs->val.a.type == 'T' && _rhs->val.a.type == 'F')
-               && !(_lhs->val.a.type == 'F' && _rhs->val.a.type == 'T'))
-                rval = (_lhs->val.a.type > _rhs->val.a.type) ? 1 : -1;
-            else
-            {
-                // the arg vals differ in this array => compare and return
-                rval = rtosc_arg_vals_cmp(_lhs+1, _rhs+1, llen, rlen, opt);
-            }
-            break;
-        }
-        case '-':
-            assert(false);
-            break;
-    }
-    else
-    {
-        rval = (_lhs->type > _rhs->type) ? 1 : -1;
-    }
-
-    return rval;
-
-#undef mfabs
-#undef cmp_3way
-}
-
-int rtosc_arg_vals_cmp(const rtosc_arg_val_t* lhs, const rtosc_arg_val_t* rhs,
-                       size_t lsize, size_t rsize,
-                       const rtosc_cmp_options* opt)
-{
-    // used if the value of lhs or rhs is range-computed:
-    rtosc_arg_val_t rlhs, rrhs;
-
-    rtosc_arg_val_t_const_itr litr, ritr;
-    rtosc_arg_val_itr_init(&litr, lhs);
-    rtosc_arg_val_itr_init(&ritr, rhs);
-
-    int rval = 0;
-
-    if(!opt)
-        opt = &default_cmp_options;
-
-    for( ; rtosc_arg_vals_cmp_has_next(&litr, &ritr, lsize, rsize) && !rval;
-        rtosc_arg_val_itr_next(&litr),
-        rtosc_arg_val_itr_next(&ritr))
-    {
-        rval = rtosc_arg_vals_cmp_single(rtosc_arg_val_itr_get(&litr, &rlhs),
-                                         rtosc_arg_val_itr_get(&ritr, &rrhs),
-                                         opt);
-    }
-
-    return rval ? rval
-                : (rtosc_arg_vals_eq_after_abort(&litr, &ritr, lsize, rsize))
-                    ? 0
-                    // they're equal until here, so one array must have
-                    // elements left:
-                    : (lsize-(litr.i) > rsize-(ritr.i))
-                        ? 1
-                        : -1;
-}
-
 void rtosc_v2args(rtosc_arg_t* args, size_t nargs, const char* arg_str,
                   rtosc_va_list_t* ap)
 {
@@ -625,10 +330,10 @@ void rtosc_v2args(rtosc_arg_t* args, size_t nargs, const char* arg_str,
                 args[arg_pos++].f = va_arg(ap->a, double);
                 break;
             case 'T':
+                args[arg_pos++].T = 1;
+                break;
             case 'F':
-            case 'N':
-            case 'I':
-                args[arg_pos++].T = arg_str[-1];
+                args[arg_pos++].T = 0;
                 break;
             default:
                 ;
@@ -673,7 +378,7 @@ size_t rtosc_vmessage(char   *buffer,
     if(!nargs)
         return rtosc_amessage(buffer,len,address,arguments,NULL);
 
-    rtosc_arg_t args[nargs];
+    STACKALLOC(rtosc_arg_t, args, nargs);
     rtosc_va_list_t ap2;
     va_copy(ap2.a, ap);
     rtosc_v2args(args, nargs, arguments, &ap2);
@@ -681,6 +386,41 @@ size_t rtosc_vmessage(char   *buffer,
     return rtosc_amessage(buffer,len,address,arguments,args);
 }
 
+size_t rtosc_avmessage(char                  *buffer,
+                       size_t                 len,
+                       const char            *address,
+                       size_t                 nargs,
+                       const rtosc_arg_val_t *args)
+{
+    rtosc_arg_val_itr itr;
+    rtosc_arg_val_itr_init(&itr, args);
+
+    int val_max;
+    {
+        // equivalent to the for loop below, in order to
+        // find out the array size
+        rtosc_arg_val_itr itr2 = itr;
+        for(val_max = 0; itr2.i < nargs; ++val_max)
+            rtosc_arg_val_itr_next(&itr2);
+    }
+
+    STACKALLOC(rtosc_arg_t, vals, val_max);
+    STACKALLOC(char, argstr,val_max+1);
+
+    int i;
+    for(i = 0; i < val_max; ++i)
+    {
+        rtosc_arg_val_t av_buffer;
+        const rtosc_arg_val_t* cur = rtosc_arg_val_itr_get(&itr, &av_buffer);
+        vals[i] = cur->val;
+        argstr[i] = cur->type;
+        rtosc_arg_val_itr_next(&itr);
+    }
+    argstr[i] = 0;
+
+    return rtosc_amessage(buffer, len, address, argstr, vals);
+}
+
 size_t rtosc_amessage(char              *buffer,
                       size_t             len,
                       const char        *address,
