@@ -3,6 +3,8 @@
 #include <rtosc/pretty-format.h>
 #include "common.h"
 
+rtosc_arg_val_t scanned[32];
+
 /**
  * @brief check_alt like check, but specify an alternative expectation
  * @see check()
@@ -20,32 +22,36 @@ void check_alt(const char* arg_val_str, const rtosc_print_options* opt,
     char tc_full[tc_len]; // descr. for full testcase name
     int strbuflen = 256;
     char strbuf[strbuflen];
+    memset(strbuf, 0x7f, strbuflen); /* init with rubbish */
 
     int num = rtosc_count_printed_arg_vals(arg_val_str);
-    assert(num > 0);
-    rtosc_arg_val_t scanned[num];
+    assert(num < 32);
+    // note: when using this, the next step is usually
+    //       rtosc_arg_val_t scanned[num];
+    //       in this case, however, we need the variable globally accessible
     size_t rd = rtosc_scan_arg_vals(arg_val_str, scanned, num,
                                     strbuf, strbuflen);
 
     strcpy(tc_full, "scan \"");
-    strncat(tc_full, tc_base, tc_len);
+    strncat(tc_full, tc_base, tc_len-7);
     strncat(tc_full, "\" (read exactly the input string)",
             tc_len - strlen(tc_full));
     assert_int_eq(strlen(arg_val_str), rd, tc_full, line);
 
     size_t len = 128;
     char printed[len];
+    memset(printed, 0x7f, len); /* init with rubbish */
     size_t written = rtosc_print_arg_vals(scanned, num, printed, len, opt, 0);
 
     const char* exp_print = _exp_print ? _exp_print : arg_val_str;
     strcpy(tc_full, "print \"");
-    strncat(tc_full, tc_base, tc_len);
+    strncat(tc_full, tc_base, tc_len-8);
     strncat(tc_full, "\" (value = value before scan)",
             tc_len - strlen(tc_full));
     assert_str_eq(exp_print, printed, tc_full, line);
 
     strcpy(tc_full, "print \"");
-    strncat(tc_full, tc_base, tc_len);
+    strncat(tc_full, tc_base, tc_len-8);
     strncat(tc_full, "\" (return value check)", tc_len - strlen(tc_full));
     assert_int_eq(strlen(exp_print), written, tc_full, line);
 }
@@ -280,8 +286,10 @@ void ranges()
     check_alt("3 ... 4", &uncompressed,
               "a very short range, uncompressed", __LINE__, "3 4");
     check("3 ... 4", &simplefloats, "a very short range, compressed", __LINE__);
-    check("2.00 1.40 ... -0.40 -1.00", &simplefloats,
-          "a simple float range", __LINE__);
+    check_alt("2.00 1.40 ... -0.40 -1.00", &simplefloats,
+              "a simple float range", __LINE__,
+              "2.00 1.40 0.80 ... -0.40 -1.00"); // TODO: bug: input != output
+                                                 //       fix in later commit?
     check_alt("'z' 'x' ... 'r'", &uncompressed,
               "a simple downward char range", __LINE__,
               "'z' 'x'\n"
@@ -301,10 +309,10 @@ void ranges()
               "    15 17\n"
               "    19");
     check_alt("[ 1 ... 3 ]", &uncompressed,
-              "range with delta 1 in an array (1)", __LINE__,
+              "range with delta 1 in an array", __LINE__,
               "[1 2 3]");
     check_alt("[3...0]", &uncompressed,
-              "range with delta 1 in an array (2)", __LINE__,
+              "range with delta -1 in an array", __LINE__,
               "[3 2 1 0]");
 
     /*
@@ -315,6 +323,11 @@ void ranges()
               __LINE__, "[0.420\n    0.420]");
     check_alt("2x[0 1]", &uncompressed,
               "range of arrays", __LINE__, "[0 1] [0\n    1]");
+    check("[127 104 64 106 7x64 101 64 64 64 92 112x64]", NULL,
+          "long array", __LINE__);
+    check_alt("[127 104 64 106 64 64 64 64 64 64 64 101 64 64 64 92 112x64]", NULL,
+          "long array", __LINE__, "[127 104 64 106 7x64 101 64 64 64 92 112x64]");
+
     /*
         combined
      */
@@ -332,17 +345,96 @@ void ranges()
 #endif
 
     /*
-        endless ranges
+        infinite ranges
      */
-    check("[1 ...]", NULL, "delta-less infinite range (1)", __LINE__);
-    check("[\"Next Effect\"S ...]", NULL,
+    check("[1 ... ]", NULL, "delta-less infinite range (1)", __LINE__);
+    check("[\"Next Effect\"S ... ]", NULL,
           "delta-less infinite range (2)", __LINE__);
     check_alt("[false...]", NULL, "delta-less infinite range (3)", __LINE__,
-              "[false ...]");
-    check("[1 0 0 ...]", NULL, "delta-less infinite range (4)", __LINE__);
-    check("[0 1 ...]", NULL, "infinite range with delta", __LINE__);
-    check("[true false false ...]", NULL,
+              "[false ... ]");
+    check("[1 0 0 ... ]", NULL, "delta-less infinite range (4)", __LINE__);
+    check("[0 1 ... ]", NULL, "infinite range with delta", __LINE__);
+    check("[true false false ... ]", NULL,
           "endless range after \"true false false\"", __LINE__);
+    check("[[0 1] ... ]", NULL, "range of arrays", __LINE__);
+    check("[true false ... ]", NULL, "alternating true-false-range", __LINE__);
+    assert('-' == scanned[2].type);
+    assert_int_eq(0, scanned[2].val.r.num,
+                  "true-false range is inifinite", __LINE__);
+    assert_int_eq(1, scanned[2].val.r.has_delta,
+                  "true-false range has delta", __LINE__);
+}
+
+// most tests here are reverse to those in ranges()
+void scan_ranges()
+{
+    /*
+        no range conversion
+     */
+    rtosc_print_options uncompressed = ((rtosc_print_options) { false, 3, " ",
+                                                                80, false });
+    rtosc_print_options simplefloats = ((rtosc_print_options) { false, 2, " ",
+                                                                80, true });
+    check("0 1 2 3", NULL, "too less args for range conversion", __LINE__);
+    check("0.00 1.00 2.00 3.00 4.00", &simplefloats,
+          "wrong type for range conversion", __LINE__);
+    check("0 1 2 3 4", &uncompressed,
+          "no range conversion due to opts", __LINE__);
+
+    /*
+        with delta
+     */
+    check_alt("1 2 3 4 5 6 7", NULL,
+              "convert to simple upwards integer range", __LINE__,
+              "1 ... 7");
+    check_alt("-3 -4 -5 -6 -7", NULL,
+              "convert to simple downwards integer range", __LINE__,
+              "-3 ... -7");
+    check_alt("'z' 'x' 'v' 't' 'r'", NULL,
+              "convert to simple downward char range", __LINE__,
+              "'z' 'x' ... 'r'");
+    check_alt("[4 3 2 1 0]", NULL,
+              "convert to downward range in an array", __LINE__,
+              "[4 ... 0]");
+    check_alt("1 3 5 7 9 13 15 17 19 21", NULL,
+              "convert to two almost subsequent ranges", __LINE__,
+              "1 3 ... 9 13 15 ... 21");
+    check_alt("[ 1 2 3 4 5 ]", NULL,
+              "convert to range with delta 1 in an array", __LINE__,
+              "[1 ... 5]");
+    check_alt("[5 4 3 2 1 0]", NULL,
+              "convert to range with delta -1 in an array", __LINE__,
+              "[5 ... 0]");
+
+    /*
+        without delta
+     */
+    check_alt("0 0 0 0 0", NULL,
+              "convert to range of 5 zeros", __LINE__, "5x0");
+    check_alt("[ 0h 0h 0h 0h 0h ]", NULL,
+              "convert to array with range of 5 huge ints", __LINE__, "[5x0h]");
+    check_alt("[0 1] [0 1] [0 1] [0 1] [0 1]", NULL,
+              "convert to range of arrays", __LINE__, "5x[0 1]");
+    /*
+        combined
+     */
+    check_alt("0.5 0.5 0.5 0.5 0.5 'a' 'b' 'c' 'd' 'e'", NULL,
+              "convert to combined ranges", __LINE__,
+              "5x0.50 (0x1p-1) 'a' ... 'e'");
+    /* combined, but not yet implemented: */
+#if 0
+    check_alt("-6 -6 -6 -6 -6 -6 -5 -4 ... 0", NULL,
+              "convert to combined ranges (2)", __LINE__,
+              "6x-6 -5 ... 0");
+    check_alt("'a' 'b' 'c' 'd' 'e' 'f' 'f' 'f' 'f' 'f' 'f'", NULL,
+              "convert to combined ranges (3)", __LINE__,
+              "'a' ... 'f' 6x'f'");
+#endif
+
+    /*
+        endless ranges
+     */
+    // (you cannot convert fixed size ranges to endless ranges)
 }
 
 void fail_at_arg(const char* arg_val_str, int exp_fail, int line)
@@ -353,21 +445,24 @@ void fail_at_arg(const char* arg_val_str, int exp_fail, int line)
     int num = rtosc_count_printed_arg_vals(arg_val_str);
 
     strcpy(tc_full, "find 1st invalid arg in \"");
-    strncat(tc_full, arg_val_str, tc_len);
+    strncat(tc_full, arg_val_str, tc_len-25);
     strncat(tc_full, "\"", tc_len);
     assert_int_eq(exp_fail, -num, tc_full, line);
 }
 
 void messages()
 {
+
     int num = rtosc_count_printed_arg_vals_of_msg("not beginning with a '/'");
     assert_int_eq(-1, num, "return -1 if the message does not start"
                            "with a slash", __LINE__);
 
     int strbuflen = 256;
     char strbuf[strbuflen];
+    memset(strbuf, 0x7f, strbuflen); /* init with rubbish */
     int msgbuflen = 256;
     char msgbuf[msgbuflen];
+    memset(msgbuf, 0x7f, msgbuflen); /* init with rubbish */
     const char* input = "%this is a savefile\n"
                         "/noteOn 0 0 0 % a noteOn message";
 
@@ -383,6 +478,7 @@ void messages()
 
     size_t len = 128;
     char printed[len];
+    memset(printed, 0x7f, len); /* init with rubbish */
     rtosc_print_options shortline = ((rtosc_print_options) { true, 3, " ", 7,
                                                              true });
     size_t written = rtosc_print_message("/noteOn", scanned, num,
@@ -558,6 +654,7 @@ int main()
     scan_and_print_single();
     scan_and_print_mulitple();
     arrays();
+    scan_ranges();
     ranges();
 
     messages();
