@@ -46,7 +46,8 @@ struct Rt:public RtData
     }
     void reply(const char *path, const char *args, ...)
     {
-        if(strcmp(path, "undo_change") || !enable)
+        //printf("# reply <%s, %s>\n", path, args);
+        if(strcmp(path, "/undo_change") || !enable)
             return;
 
         va_list va;
@@ -99,7 +100,7 @@ int main()
     assert_int_eq(7, o.b,
             "Verify Redo Has Returned To Altered State", __LINE__);
 
-    return 0;
+    return test_summary();
 }
 
 
