@@ -20,6 +20,8 @@
 
 #define MAX_ALIENWAH_DELAY 100
 
+namespace zyn {
+
 /**"AlienWah" Effect*/
 class Alienwah:public Effect
 {
@@ -58,4 +60,6 @@ class Alienwah:public Effect
         int oldk;
 };
 
+}
+
 #endif
