@@ -17,6 +17,8 @@
 #include "Effect.h"
 #include "EffectLFO.h"
 
+namespace zyn {
+
 /**DynamicFilter Effect*/
 class DynamicFilter:public Effect
 {
@@ -56,4 +58,6 @@ class DynamicFilter:public Effect
         float ms1, ms2, ms3, ms4; //mean squares
 };
 
+}
+
 #endif
