@@ -16,6 +16,8 @@
 
 #include "Effect.h"
 
+namespace zyn {
+
 /**EQ Effect*/
 class EQ:public Effect
 {
@@ -53,4 +55,6 @@ class EQ:public Effect
         } filter[MAX_EQ_BANDS];
 };
 
+}
+
 #endif
