@@ -14,6 +14,8 @@
 #ifndef EFFECT_LFO_H
 #define EFFECT_LFO_H
 
+namespace zyn {
+
 /**LFO for some of the Effect objects
  * \todo see if this should inherit LFO*/
 class EffectLFO
@@ -41,4 +43,6 @@ class EffectLFO
         float buffersize_f;
 };
 
+}
+
 #endif
