@@ -23,6 +23,8 @@
 
 #define MAX_PHASER_STAGES 12
 
+namespace zyn {
+
 class Phaser:public Effect
 {
     public:
@@ -87,4 +89,6 @@ class Phaser:public Effect
         float applyPhase(float x, float g, float *old);
 };
 
+}
+
 #endif
