@@ -114,7 +114,9 @@ static const Ports sysefsendto =
         }}
 };
 
-#define rBegin [](const char *msg, RtData &d) { rtosc::AutomationMgr &a = *(AutomationMgr*)d.obj
+#define rBegin [](const char *msg, RtData &d) { \
+    (void) msg; \
+    rtosc::AutomationMgr &a = *(AutomationMgr*)d.obj
 #define rEnd }
 
 static int extract_num(const char *&msg)
@@ -172,11 +174,17 @@ static const Ports auto_param_ports = {
         else
             d.reply(d.loc, a.slots[slot].automations[param].active ? "T" : "F");
         rEnd},
-    {"path::s", rProp(parameter) rProp(read-only) rDoc("Path of parameter"), 0,
+    {"path::s", rProp(parameter) rDoc("Path of parameter"), 0,
         rBegin;
         int slot  = d.idx[1];
         int param = d.idx[0];
-        d.reply(d.loc, "s", a.slots[slot].automations[param].param_path);
+        if(!strcmp("s",rtosc_argument_string(msg))) {
+            a.setSlotSubPath(slot, param, rtosc_argument(msg, 0).s);
+            a.updateMapping(slot, param);
+            d.broadcast(d.loc, "s", a.slots[slot].automations[param].param_path);
+        }
+        else
+			d.reply(d.loc, "s", a.slots[slot].automations[param].param_path);
         rEnd},
     {"clear:", rDoc("Clear automation param"), 0,
         rBegin;
@@ -242,9 +250,10 @@ static const Ports slot_ports = {
     {"midi-cc::i", rProp(parameter) rMap(default, -1) rDoc("Access assigned midi CC slot") , 0,
         rBegin;
         int slot = d.idx[0];
-        if(rtosc_narguments(msg))
+        if(rtosc_narguments(msg)) {
             a.slots[slot].midi_cc = rtosc_argument(msg, 0).i;
-        else
+            d.broadcast(d.loc, "i", a.slots[slot].midi_cc);
+        } else
             d.reply(d.loc, "i", a.slots[slot].midi_cc);
 
         rEnd},
@@ -298,8 +307,7 @@ static const Ports automate_ports = {
         if(a.active_slot >= 0)
             a.createBinding(a.active_slot, rtosc_argument(msg, 0).s, true);
         rEnd},
-    // TODO: remove rNoWalk
-    {"slot#16/", rNoWalk rDoc("Parameters of individual automation slots"), &slot_ports,
+    {"slot#16/", rDoc("Parameters of individual automation slots"), &slot_ports,
         rBegin;
         (void)a;
         d.push_index(get_next_int(msg));
@@ -380,12 +388,12 @@ static const Ports master_ports = {
        d.reply(m-1);}},
     {"get-vu:", rDoc("Grab VU Data"), 0, [](const char *, RtData &d) {
        Master *m = (Master*)d.obj;
-       d.reply("/vu-meter", "bb", sizeof(m->vu), &m->vu, sizeof(float)*NUM_MIDI_PARTS, m->vuoutpeakpart);}},
+       d.reply("/vu-meter", "bb", sizeof(m->vu), &m->vu, sizeof(float)*NUM_MIDI_PARTS, m->vuoutpeakpartl);}},
     {"vu-meter:", rDoc("Grab VU Data"), 0, [](const char *, RtData &d) {
        Master *m = (Master*)d.obj;
-       char        types[6+NUM_MIDI_PARTS+1] = {0};
-       rtosc_arg_t  args[6+NUM_MIDI_PARTS+1];
-       for(int i=0; i<6+NUM_MIDI_PARTS; ++i)
+       char        types[6+2*NUM_MIDI_PARTS+1] = {0};
+       rtosc_arg_t  args[6+2*NUM_MIDI_PARTS+1];
+       for(int i=0; i<6+2*NUM_MIDI_PARTS; ++i)
            types[i] = 'f';
        args[0].f = m->vu.outpeakl;
        args[1].f = m->vu.outpeakr;
@@ -393,9 +401,10 @@ static const Ports master_ports = {
        args[3].f = m->vu.maxoutpeakr;
        args[4].f = m->vu.rmspeakl;
        args[5].f = m->vu.rmspeakr;
-       for(int i=0; i<NUM_MIDI_PARTS; ++i)
-           args[6+i].f = m->vuoutpeakpart[i];
-
+       for(int i=0; i<NUM_MIDI_PARTS; ++i) {
+           args[6 + 2 * i].f = m->vuoutpeakpartl[i];
+           args[6 + 2 * i + 1].f = m->vuoutpeakpartr[i];
+       }
        d.replyArray("/vu-meter", types, args);}},
     {"reset-vu:", rDoc("Grab VU Data"), 0, [](const char *, RtData &d) {
        Master *m = (Master*)d.obj;
@@ -409,8 +418,7 @@ static const Ports master_ports = {
        d.reply("/free", "sb", "Part", sizeof(void*), &m->part[i]);
        m->part[i] = p;
        p->initialize_rt();
-       for(int i=0; i<128; ++i)
-           m->activeNotes[i] = 0;
+       memset(m->activeNotes, 0, sizeof(m->activeNotes));
        }},
     {"active_keys:", rProp("Obtain a list of active notes"), 0,
         rBegin;
@@ -423,18 +431,23 @@ static const Ports master_ports = {
         rDefault(80) rDoc("Master Volume"), 0,
         [](const char *m, rtosc::RtData &d) {
         if(rtosc_narguments(m)==0) {
-            d.reply(d.loc, "i", ((Master*)d.obj)->Pvolume);
+            d.reply(d.loc, "i", (int) roundf(96.0f * ((Master*)d.obj)->Volume / 40.0f + 96.0f));
         } else if(rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
-            ((Master*)d.obj)->setPvolume(limit<char>(rtosc_argument(m,0).i,0,127));
-            d.broadcast(d.loc, "i", ((Master*)d.obj)->Pvolume);}}},
+            ((Master *)d.obj)->Volume  = ((Master *)d.obj)->volume127ToFloat(limit<unsigned char>(rtosc_argument(m, 0).i, 0, 127));
+             d.broadcast(d.loc, "i", limit<char>(rtosc_argument(m, 0).i, 0, 127));
+        }}},
     {"volume::i", rShort("volume") rProp(parameter) rLinear(0,127)
-        rDefault(80) rDoc("Master Volume"), 0,
-        [](const char *m, rtosc::RtData &d) {
-        if(rtosc_narguments(m)==0) {
-            d.reply(d.loc, "i", ((Master*)d.obj)->Pvolume);
-        } else if(rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
-            ((Master*)d.obj)->setPvolume(limit<char>(rtosc_argument(m,0).i,0,127));
-            d.broadcast(d.loc, "i", ((Master*)d.obj)->Pvolume);}}},
+           rDoc("Master Volume"), 0,
+           [](const char *m, rtosc::RtData &d) {
+           Master *master = (Master *)d.obj;
+           if(rtosc_narguments(m)==0) {
+                d.reply(d.loc, "i", (int) roundf(96.0f * master->Volume / 40.0f + 96.0f));
+            } else if (rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
+               master->Volume  = master->volume127ToFloat(limit<unsigned char>(rtosc_argument(m, 0).i, 0, 127));
+               d.broadcast(d.loc, "i", limit<char>(rtosc_argument(m, 0).i, 0, 127));
+           }}},
+    rParamF(Volume, rShort("volume"), rDefault(-6.66667f), rLinear(-40.0f,13.3333f),
+             rUnit(dB), "Master Volume"),
     {"Psysefxvol#" STRINGIFY(NUM_SYS_EFX) "/::i", 0, &sysefxPort,
         [](const char *msg, rtosc::RtData &d) {
             SNIP;
@@ -513,7 +526,8 @@ static const Ports master_ports = {
         [](const char *, rtosc::RtData &d) {d.reply("/undo_pause", "");}},
     {"undo_resume:",rProp(internal) rDoc("resume undo event recording"),0,
         [](const char *, rtosc::RtData &d) {d.reply("/undo_resume", "");}},
-    {"config/", rNoWalk rDoc("Top Level Application Configuration Parameters"),
+    {"config/", rNoDefaults
+        rDoc("Top Level Application Configuration Parameters"),
         &Config::ports, [](const char *, rtosc::RtData &d){d.forward();}},
     {"presets/", rDoc("Parameter Presets"), &preset_ports, rBOIL_BEGIN
         SNIP
@@ -605,6 +619,7 @@ class DataObj:public rtosc::RtData
         virtual void forward(const char *reason) override
         {
             assert(message);
+            (void) reason;
             reply("/forward", "");
             printf("forwarding '%s'\n", message);
             forwarded = true;
@@ -636,6 +651,7 @@ void Master::saveAutomation(XMLwrapper &xml, const rtosc::AutomationMgr &midi)
             xml.beginbranch("slot", i);
             XmlNode params("params");
             params["midi-cc"] = to_s(slot.midi_cc);
+            params["name"] = to_s(slot.name);
             xml.add(params);
             for(int j=0; j<midi.per_slot; ++j) {
                 const auto &au = slot.automations[j];
@@ -681,12 +697,21 @@ void Master::loadAutomation(XMLwrapper &xml, rtosc::AutomationMgr &midi)
                         midi.createBinding(i, path.c_str(), false);
                         midi.setSlotSubGain(i, j, gain);
                         midi.setSlotSubOffset(i, j, offset);
+                        midi.updateMapping(i, j);
                         xml.exitbranch();
                     }
                 }
                 for(auto node:xml.getBranch())
+                {
                     if(node.name == "params")
+                    {
                         slot.midi_cc = atoi(node["midi-cc"].c_str());
+                        if(node["name"] != "")
+                        {
+                            strncpy(slot.name, node["name"].c_str(), sizeof(slot.name) - 1);
+                        }
+                    }
+                }
                 xml.exitbranch();
             }
         }
@@ -722,7 +747,8 @@ Master::Master(const SYNTH_T &synth_, Config* config)
 
     shutup = 0;
     for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart) {
-        vuoutpeakpart[npart] = 1e-9;
+        vuoutpeakpartl[npart] = 1e-9;
+        vuoutpeakpartr[npart] = 1e-9;
         fakepeakpart[npart]  = 0;
     }
 
@@ -741,8 +767,7 @@ Master::Master(const SYNTH_T &synth_, Config* config)
         sysefx[nefx] = new EffectMgr(*memory, synth, 0, &time);
 
     //Note Visualization
-    for(int i=0; i<128; ++i)
-        activeNotes[i] = 0;
+    memset(activeNotes, 0, sizeof(activeNotes));
 
     defaults();
 
@@ -834,8 +859,10 @@ bool Master::applyOscEvent(const char *msg, bool nio, int msg_id)
 
 void Master::defaults()
 {
-    volume = 1.0f;
-    setPvolume(80);
+    union {float f; uint32_t i;} convert;
+    convert.i = 0xC0D55556;
+    Volume = convert.f;
+    oldVolume = Volume;
     setPkeyshift(64);
 
     for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart) {
@@ -868,38 +895,38 @@ void Master::defaults()
 /*
  * Note On Messages (velocity=0 for NoteOff)
  */
-void Master::noteOn(char chan, char note, char velocity)
+void Master::noteOn(char chan, note_t note, char velocity, float note_log2_freq)
 {
     if(velocity) {
         for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart) {
             if(chan == part[npart]->Prcvchn) {
                 fakepeakpart[npart] = velocity * 2;
                 if(part[npart]->Penabled)
-                    part[npart]->NoteOn(note, velocity, keyshift);
+                    part[npart]->NoteOn(note, velocity, keyshift, note_log2_freq);
             }
         }
-        activeNotes[(int)note] = 1;
+        activeNotes[note] = 1;
+        HDDRecorder.triggernow();
     }
     else
         this->noteOff(chan, note);
-    HDDRecorder.triggernow();
 }
 
 /*
  * Note Off Messages
  */
-void Master::noteOff(char chan, char note)
+void Master::noteOff(char chan, note_t note)
 {
     for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart)
         if((chan == part[npart]->Prcvchn) && part[npart]->Penabled)
             part[npart]->NoteOff(note);
-    activeNotes[(int)note] = 0;
+    activeNotes[note] = 0;
 }
 
 /*
  * Pressure Messages (velocity=0 for NoteOff)
  */
-void Master::polyphonicAftertouch(char chan, char note, char velocity)
+void Master::polyphonicAftertouch(char chan, note_t note, char velocity)
 {
     if(velocity) {
         for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart)
@@ -951,7 +978,7 @@ void Master::setController(char chan, int type, int par)
     }
 }
 
-void Master::vuUpdate(const float *outl, const float *outr)
+void Master::vuUpdate(const float *outr, const float *outl)
 {
     //Peak computation (for vumeters)
     vu.outpeakl = 1e-12;
@@ -981,16 +1008,17 @@ void Master::vuUpdate(const float *outl, const float *outr)
 
     //Part Peak computation (for Part vumeters or fake part vumeters)
     for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart) {
-        vuoutpeakpart[npart] = 1.0e-12f;
+        vuoutpeakpartl[npart] = 1.0e-12f;
+        vuoutpeakpartr[npart] = 1.0e-12f;
         if(part[npart]->Penabled != 0) {
-            float *outl = part[npart]->partoutl,
-            *outr = part[npart]->partoutr;
+            float *outr = part[npart]->partoutl,
+            *outl = part[npart]->partoutr;
             for(int i = 0; i < synth.buffersize; ++i) {
-                float tmp = fabs(outl[i] + outr[i]);
-                if(tmp > vuoutpeakpart[npart])
-                    vuoutpeakpart[npart] = tmp;
+                if (fabs(outl[i]) > vuoutpeakpartl[npart])
+                    vuoutpeakpartl[npart] = fabs(outl[i]);
+                if (fabs(outr[i]) > vuoutpeakpartr[npart])
+                    vuoutpeakpartr[npart] = fabs(outr[i]);
             }
-            vuoutpeakpart[npart] *= volume;
         }
         else
         if(fakepeakpart[npart] > 1)
@@ -1272,10 +1300,25 @@ bool Master::AudioOut(float *outr, float *outl)
 
 
     //Master Volume
-    for(int i = 0; i < synth.buffersize; ++i) {
-        outl[i] *= volume;
-        outr[i] *= volume;
+    float oldvol = dB2rap(oldVolume);
+    float newvol = dB2rap(Volume);
+    if(ABOVE_AMPLITUDE_THRESHOLD(oldvol, newvol)) {
+        for(int i = 0; i < synth.buffersize; ++i) {
+            float vol = INTERPOLATE_AMPLITUDE(oldvol, newvol,
+                                              i, synth.buffersize);
+            outl[i] *= vol;
+            outr[i] *= vol;
+        }
     }
+    else {
+        // No interpolation
+        float vol = dB2rap(Volume);
+        for(int i = 0; i < synth.buffersize; ++i) {
+            outl[i] *= vol;
+            outr[i] *= vol;
+        }
+    }
+    oldVolume = Volume;
 
     vuUpdate(outl, outr);
 
@@ -1371,10 +1414,10 @@ Master::~Master()
 /*
  * Parameter control
  */
-void Master::setPvolume(char Pvolume_)
+
+float Master::volume127ToFloat(unsigned char volume_)
 {
-    Pvolume = Pvolume_;
-    volume  = dB2rap((Pvolume - 96.0f) / 96.0f * 40.0f);
+    return (volume_ - 96.0f) / 96.0f * 40.0;
 }
 
 void Master::setPkeyshift(char Pkeyshift_)
@@ -1410,8 +1453,7 @@ void Master::ShutUp()
         insefx[nefx]->cleanup();
     for(int nefx = 0; nefx < NUM_SYS_EFX; ++nefx)
         sysefx[nefx]->cleanup();
-    for(int i = 0; i < int(sizeof(activeNotes)/sizeof(activeNotes[0])); ++i)
-        activeNotes[i] = 0;
+    memset(activeNotes, 0, sizeof(activeNotes));
     vuresetpeaks();
     shutup = 0;
 }
@@ -1448,7 +1490,7 @@ void Master::initialize_rt(void)
 
 void Master::add2XML(XMLwrapper& xml)
 {
-    xml.addpar("volume", Pvolume);
+    xml.addparreal("volume", Volume);
     xml.addpar("key_shift", Pkeyshift);
     xml.addparbool("nrpn_receive", ctl.NRPN.receive);
 
@@ -1564,7 +1606,12 @@ int Master::loadXML(const char *filename)
 
 void Master::getfromXML(XMLwrapper& xml)
 {
-    setPvolume(xml.getpar127("volume", Pvolume));
+    if (xml.hasparreal("volume")) {
+        xml.getparreal("volume", Volume);
+    } else {
+        Volume  = volume127ToFloat(xml.getpar127("volume", 0));
+        oldVolume = Volume;
+    }
     setPkeyshift(xml.getpar127("key_shift", Pkeyshift));
     ctl.NRPN.receive = xml.getparbool("nrpn_receive", ctl.NRPN.receive);
 
