@@ -51,6 +51,7 @@ const rtosc::Ports Microtonal::ports = {
     rParamZyn(PAnote, rShort("1/1 midi note"), rDefault(69),
         "The note for 'A'"),
     rParamF(PAfreq,   rShort("ref freq"), rDefault(440.0f),
+         rLog(1.0,20000.0),
         "Frequency of the 'A' note"),
     rParamZyn(Pscaleshift, rShort("shift"), rDefault(64),
         "UNDOCUMENTED"),
@@ -65,7 +66,7 @@ const rtosc::Ports Microtonal::ports = {
     rParamZyn(Pmapsize, rDefault(12), "Size of key map"),
     rToggle(Pmappingenabled, rDefault(false), "Mapping Enable"),
 
-    rParams(Pmapping, 128, rDefaultMissing, "Mapping of keys"),
+    rParams(Pmapping, 128, rDefault([0 1 ...]), "Mapping of keys"),
     rParamZyn(Pglobalfinedetune, rShort("fine"), rDefault(64),
         "Fine detune for all notes"),
 
@@ -256,8 +257,10 @@ unsigned char Microtonal::getoctavesize() const
 /*
  * Get the frequency according the note number
  */
-float Microtonal::getnotefreq(int note, int keyshift) const
+float Microtonal::getnotefreq(float note_log2_freq, int keyshift) const
 {
+    note_t note = roundf(12.0f * note_log2_freq);
+
     // in this function will appears many times things like this:
     // var=(a+b*100)%b
     // I had written this way because if I use var=a%b gives unwanted results when a<0
@@ -271,9 +274,8 @@ float Microtonal::getnotefreq(int note, int keyshift) const
         powf(2.0f, (Pglobalfinedetune - 64.0f) / 1200.0f);       //-64.0f .. 63.0f cents
 
     if(Penabled == 0) //12tET
-        return powf(2.0f,
-                    (note - PAnote
-                     + keyshift) / 12.0f) * PAfreq * globalfinedetunerap;
+        return powf(2.0f, note_log2_freq +
+                    ((keyshift - PAnote) / 12.0f)) * PAfreq * globalfinedetunerap;
 
     int scaleshift =
         ((int)Pscaleshift - 64 + (int) octavesize * 100) % octavesize;
