@@ -18,6 +18,8 @@
 #include "../globals.h"
 #include "Engine.h"
 
+namespace zyn {
+
 class AudioOut:public virtual Engine
 {
     public:
@@ -50,4 +52,6 @@ class AudioOut:public virtual Engine
         int bufferSize;
 };
 
+}
+
 #endif
