@@ -31,10 +31,12 @@
 #include "OssMultiEngine.h"
 #include "Compressor.h"
 
-extern MiddleWare *middleware;
-
 using namespace std;
 
+extern zyn::MiddleWare *middleware;
+
+namespace zyn {
+
 OssMultiEngine :: OssMultiEngine(const SYNTH_T &synth,
     const oss_devs_t &oss_devs)
     :AudioOut(synth),
@@ -265,3 +267,5 @@ done:
     pthread_exit(0);
     return (0);
 }
+
+}
