@@ -16,6 +16,8 @@
 #include "ZynSema.h"
 #include <pthread.h>
 
+namespace zyn {
+
 /**
  * C++ thread safe lockless queue
  * Based off of jack's ringbuffer*/
@@ -55,5 +57,7 @@ class SafeQueue
         T *buffer;
 };
 
+}
+
 #include "SafeQueue.cpp"
 #endif
