@@ -128,8 +128,28 @@ static const Ports voicePorts = {
     //Amplitude Stuff
     rParamZyn(PPanning,                  rShort("pan."), rDefault(64),
         "Panning"),
-    rParamZyn(PVolume,                   rShort("vol."), rDefault(100),
-        "Volume"),
+    {"PVolume::i", rShort("vol.") rLinear(0,127)
+        rDoc("Volume"), NULL,
+        [](const char *msg, RtData &d)
+        {
+            rObject *obj = (rObject *)d.obj;
+            if (!rtosc_narguments(msg))
+                d.reply(d.loc, "i", (int)roundf(127.0f * (1.0f + obj->volume/60.0f)));
+            else 
+                obj->volume = -60.0f * (1.0f - rtosc_argument(msg, 0).i / 127.0f);
+        }},
+    {"volume::f", rShort("volume") rProp(parameter) rUnit(dB) rDefault(-12.75) rLinear(-60.0f, 0.0f)
+        rDoc("Part Volume"), NULL,
+        [](const char *msg, RtData &d)
+        {
+            rObject *obj = (rObject *)d.obj;
+            if(!rtosc_narguments(msg)) {
+               d.reply(d.loc, "f", obj->volume);
+            } else if (rtosc_narguments(msg) == 1 && rtosc_type(msg, 0) == 'f') {
+               obj->volume = rtosc_argument(msg, 0).f;
+               d.broadcast(d.loc, "f", ((obj->volume)));
+            }
+        }},
     rToggle(PVolumeminus,                rShort("inv."), rDefault(false),
         "Signal Inverter"), //do we really need this??
     rParamZyn(PAmpVelocityScaleFunction, rShort("sense"), rDefault(127),
@@ -151,14 +171,14 @@ static const Ports voicePorts = {
     rParamZyn(PFilterVelocityScaleFunction, rShort("v.sense"), rDefault(64),
         "Filter Velocity Function Shape"),
 
-
+ 
     //Modulator Stuff
-    rOption(PFMEnabled, rShort("mode"), rOptions(none, morph, ring, phase,
+    rOption(PFMEnabled, rShort("mode"), rOptions(none, mix, ring, phase,
                 frequency, pulse), rDefault(none), "Modulator mode"),
     rParamI(PFMVoice,                   rShort("voice"), rDefault(-1),
         "Modulator Oscillator Selection"),
-    rParamZyn(PFMVolume,                rShort("vol."),  rDefault(90),
-        "Modulator Magnitude"),
+    rParamF(FMvolume,                   rShort("vol."),  rLinear(0.0, 100.0),
+        rDefault(70.0),                 "Modulator Magnitude"),
     rParamZyn(PFMVolumeDamp,            rShort("damp."), rDefault(64),
         "Modulator HF dampening"),
     rParamZyn(PFMVelocityScaleFunction, rShort("sense"), rDefault(64),
@@ -180,6 +200,7 @@ static const Ports voicePorts = {
             "Modulator Amplitude Envelope"),
 
 
+
     //weird stuff for PCoarseDetune
     {"detunevalue:",  rMap(unit,cents) rDoc("Get detune in cents"), NULL,
         [](const char *, RtData &d)
@@ -207,7 +228,7 @@ static const Ports voicePorts = {
                 obj->PCoarseDetune = k*1024 + obj->PCoarseDetune%1024;
             }
         }},
-    {"coarsedetune::c:i", rProp(parameter) rShort("coarse") rLinear(-64,63)
+    {"coarsedetune::c:i", rProp(parameter) rShort("coarse") rLinear(-64,63) rDefault(0)
         rDoc("Coarse note detune"), NULL,
         [](const char *msg, RtData &d)
         {
@@ -222,7 +243,17 @@ static const Ports voicePorts = {
                 obj->PCoarseDetune = k + (obj->PCoarseDetune/1024)*1024;
             }
         }},
-
+    {"PFMVolume::i", rShort("vol.") rLinear(0,127)
+        rDoc("Modulator Magnitude"), NULL,
+        [](const char *msg, RtData &d)
+        {
+            rObject *obj = (rObject *)d.obj;
+            if (!rtosc_narguments(msg))
+                d.reply(d.loc, "i", (int)roundf(127.0f * obj->FMvolume
+                    / 100.0f));
+            else
+                obj->FMvolume = 100.0f * rtosc_argument(msg, 0).i / 127.0f;
+        }},
     //weird stuff for PCoarseDetune
     {"FMdetunevalue:", rMap(unit,cents) rDoc("Get modulator detune"), NULL, [](const char *, RtData &d)
         {
@@ -305,10 +336,20 @@ static const Ports globalPorts = {
     //Amplitude
     rParamZyn(PPanning, rShort("pan"), rDefault(64),
         "Panning of ADsynth (0 random, 1 left, 127 right)"),
-    rParamZyn(PVolume,  rShort("vol"), rDefault(90), "volume control"),
-    rParamZyn(PAmpVelocityScaleFunction, rShort("scale"), rDefault(64),
-        "Volume Velocity Control"),
-
+    rParamF(Volume,                   rShort("vol"), rLinear(-60.0f,20.0f),
+        rUnit(dB), rDefault(-3.75f), "volume control"),
+    rParamZyn(PAmpVelocityScaleFunction, rShort("sense"), rDefault(64),
+        "Volume velocity sense"),
+    {"PVolume::i", rShort("vol.") rLinear(0,127)
+        rDoc("Volume"), NULL,
+        [](const char *msg, RtData &d)
+        {
+            rObject *obj = (rObject *)d.obj;
+            if (!rtosc_narguments(msg))
+                d.reply(d.loc, "i", (int)roundf(96.0f * (1.0f + obj->Volume/60.0f)));
+            else 
+                obj->Volume = -60.0f * (1.0f - rtosc_argument(msg, 0).i / 96.0f);
+        }},
     rParamZyn(Fadein_adjustment, rDefault(FADEIN_ADJUSTMENT_SCALE),
         "Adjustment for anti-pop strategy."),
     rParamZyn(PPunchStrength, rShort("strength"),     rDefault(0),
@@ -321,7 +362,7 @@ static const Ports globalPorts = {
         "Punch Velocity control"),
 
     //Filter
-    rParamZyn(PFilterVelocityScale,         rShort("scale"), rDefault(64),
+    rParamZyn(PFilterVelocityScale,         rShort("scale"), rDefault(0),
         "Filter Velocity Magnitude"),
     rParamZyn(PFilterVelocityScaleFunction, rShort("sense"), rDefault(64),
         "Filter Velocity Function Shape"),
@@ -450,7 +491,7 @@ void ADnoteGlobalParam::defaults()
     PBandwidth = 64;
 
     /* Amplitude Global Parameters */
-    PVolume  = 90;
+    Volume  = -3.75f;
     PPanning = 64; //center
     PAmpVelocityScaleFunction = 64;
     AmpEnvelope->defaults();
@@ -463,7 +504,7 @@ void ADnoteGlobalParam::defaults()
     Hrandgrouping = 0;
 
     /* Filter Global Parameters*/
-    PFilterVelocityScale = 64;
+    PFilterVelocityScale = 0;
     PFilterVelocityScaleFunction = 64;
     GlobalFilter->defaults();
     FilterEnvelope->defaults();
@@ -503,7 +544,7 @@ void ADnoteVoiceParam::defaults()
     Poscilphase   = 64;
     PFMoscilphase = 64;
     PDelay                    = 0;
-    PVolume                   = 100;
+    volume                    = -60.0f* (1.0f - 100.0f / 127.0f);
     PVolumeminus              = 0;
     PPanning                  = 64; //center
     PDetune                   = 8192; //8192=0
@@ -525,7 +566,7 @@ void ADnoteVoiceParam::defaults()
     //I use the internal oscillator (-1)
     PFMVoice = -1;
 
-    PFMVolume       = 90;
+    FMvolume       = 70.0;
     PFMVolumeDamp   = 64;
     PFMDetune       = 8192;
     PFMCoarseDetune = 0;
@@ -714,7 +755,7 @@ void ADnoteVoiceParam::add2XML(XMLwrapper& xml, bool fmoscilused)
 
     xml.beginbranch("AMPLITUDE_PARAMETERS");
     xml.addpar("panning", PPanning);
-    xml.addpar("volume", PVolume);
+    xml.addparreal("volume", volume);
     xml.addparbool("volume_minus", PVolumeminus);
     xml.addpar("velocity_sensing", PAmpVelocityScaleFunction);
 
@@ -789,7 +830,7 @@ void ADnoteVoiceParam::add2XML(XMLwrapper& xml, bool fmoscilused)
         xml.beginbranch("FM_PARAMETERS");
         xml.addpar("input_voice", PFMVoice);
 
-        xml.addpar("volume", PFMVolume);
+        xml.addparreal("volume", FMvolume);
         xml.addpar("volume_damp", PFMVolumeDamp);
         xml.addpar("velocity_sensing",
                     PFMVelocityScaleFunction);
@@ -829,7 +870,7 @@ void ADnoteGlobalParam::add2XML(XMLwrapper& xml)
     xml.addparbool("stereo", PStereo);
 
     xml.beginbranch("AMPLITUDE_PARAMETERS");
-    xml.addpar("volume", PVolume);
+    xml.addparreal("volume", Volume);
     xml.addpar("panning", PPanning);
     xml.addpar("velocity_sensing", PAmpVelocityScaleFunction);
     xml.addpar("fadein_adjustment", Fadein_adjustment);
@@ -904,7 +945,15 @@ void ADnoteGlobalParam::getfromXML(XMLwrapper& xml)
     PStereo = xml.getparbool("stereo", PStereo);
 
     if(xml.enterbranch("AMPLITUDE_PARAMETERS")) {
-        PVolume  = xml.getpar127("volume", PVolume);
+        const bool upgrade_3_0_3 = (xml.fileversion() < version_type(3,0,3)) ||
+            (!xml.hasparreal("volume"));
+
+        if (upgrade_3_0_3) {
+            int vol = xml.getpar127("volume", 0);
+            Volume    = -60.0f * ( 1.0f - vol / 96.0f);
+        } else {
+            Volume    = xml.getparreal("volume", Volume);
+        }
         PPanning = xml.getpar127("panning", PPanning);
         PAmpVelocityScaleFunction = xml.getpar127("velocity_sensing",
                                                    PAmpVelocityScaleFunction);
@@ -1051,7 +1100,7 @@ void ADnoteVoiceParam::paste(ADnoteVoiceParam &a)
 
 
     copy(PPanning);
-    copy(PVolume);
+    copy(volume);
     copy(PVolumeminus);
     copy(PAmpVelocityScaleFunction);
     copy(PAmpEnvelopeEnabled);
@@ -1090,7 +1139,7 @@ void ADnoteVoiceParam::paste(ADnoteVoiceParam &a)
     RCopy(FilterLfo);
 
     copy(PFMVoice);
-    copy(PFMVolume);
+    copy(FMvolume);
     copy(PFMVolumeDamp);
     copy(PFMVelocityScaleFunction);
 
@@ -1117,7 +1166,7 @@ void ADnoteGlobalParam::paste(ADnoteGlobalParam &a)
 {
     copy(PStereo);
 
-    copy(PVolume);
+    copy(Volume);
     copy(PPanning);
     copy(PAmpVelocityScaleFunction);
 
@@ -1191,7 +1240,15 @@ void ADnoteVoiceParam::getfromXML(XMLwrapper& xml, unsigned nvoice)
 
     if(xml.enterbranch("AMPLITUDE_PARAMETERS")) {
         PPanning     = xml.getpar127("panning", PPanning);
-        PVolume      = xml.getpar127("volume", PVolume);
+        const bool upgrade_3_0_3 = (xml.fileversion() < version_type(3,0,3)) ||
+            (!xml.hasparreal("volume"));
+
+        if (upgrade_3_0_3) {
+            int vol = xml.getpar127("volume", 0);
+            volume    = -60.0f * ( 1.0f - vol / 127.0f);
+        } else {
+            volume    = xml.getparreal("volume", volume);
+        }
         PVolumeminus = xml.getparbool("volume_minus", PVolumeminus);
         PAmpVelocityScaleFunction = xml.getpar127("velocity_sensing",
                                                    PAmpVelocityScaleFunction);
@@ -1264,8 +1321,16 @@ void ADnoteVoiceParam::getfromXML(XMLwrapper& xml, unsigned nvoice)
     }
 
     if(xml.enterbranch("FM_PARAMETERS")) {
+        const bool upgrade_3_0_3 = (xml.fileversion() < version_type(3,0,3)) ||
+            (xml.getparreal("volume", -1) < 0);
+
         PFMVoice      = xml.getpar("input_voice", PFMVoice, -1, nvoice - 1);
-        PFMVolume     = xml.getpar127("volume", PFMVolume);
+        if (upgrade_3_0_3) {
+            int Pvolume = xml.getpar127("volume", 0);
+            FMvolume    = 100.0f * Pvolume / 127.0f;
+        } else {
+            FMvolume    = xml.getparreal("volume", FMvolume);
+        }
         PFMVolumeDamp = xml.getpar127("volume_damp", PFMVolumeDamp);
         PFMVelocityScaleFunction = xml.getpar127("velocity_sensing",
                                                   PFMVelocityScaleFunction);
