@@ -20,7 +20,7 @@
 namespace zyn {
 
 enum FMTYPE {
-    NONE, MORPH, RING_MOD, PHASE_MOD, FREQ_MOD, PW_MOD
+    NONE, MIX, RING_MOD, PHASE_MOD, FREQ_MOD, PW_MOD
 };
 
 /*****************************************************************/
@@ -64,7 +64,7 @@ struct ADnoteGlobalParam {
             127 - right */
     unsigned char PPanning;
 
-    unsigned char PVolume;
+    float Volume;
 
     unsigned char PAmpVelocityScaleFunction;
 
@@ -214,7 +214,7 @@ struct ADnoteVoiceParam {
     unsigned char PPanning;
 
     /* Voice Volume */
-    unsigned char PVolume;
+    float volume;
 
     /* If the Volume negative */
     unsigned char PVolumeminus;
@@ -258,7 +258,7 @@ struct ADnoteVoiceParam {
     *   MODULLATOR PARAMETERS   *
     ****************************/
 
-    /* Modullator Parameters (0=off,1=Morph,2=RM,3=PM,4=FM.. */
+    /* Modullator Parameters (0=off,1=Mix,2=RM,3=PM,4=FM.. */
     unsigned char PFMEnabled;
 
     /* Voice that I use as modullator instead of FMSmp.
@@ -270,7 +270,7 @@ struct ADnoteVoiceParam {
     OscilGen *FMSmp;
 
     /* Modullator Volume */
-    unsigned char PFMVolume;
+    float FMvolume;
 
     /* Modullator damping at higher frequencies */
     unsigned char PFMVolumeDamp;
