@@ -44,9 +44,9 @@ const rtosc::Ports Controller::ports = {
     rToggle(modwheel.exponential,  rShort("mdw.exp"), rDefault(false),
         "Modwheel Exponential Mode"),
     rToggle(pitchwheel.is_split,   rDefault(false),
-        "If PitchWheel Has unified bendrange or not"),
+        "If PitchWheel has unified bendrange or not"),
     rParamI(pitchwheel.bendrange,  rShort("pch.d"), rDefault(200),
-            rLinear(-6400, 6400),
+            rLinear(-6400, 6400), rUnit(% of semitone),
         "Range of MIDI Pitch Wheel"),
     rParamI(pitchwheel.bendrange_down, rDefault(0),
         "Lower Range of MIDI Pitch Wheel"),
