@@ -43,6 +43,41 @@ class Presets
         //virtual void defaults() = 0;
 };
 
+/*
+    Location where a "consumer" in zyn is located, where
+    consumers are envelopes, LFOs and filters.
+    Note that the AD synth global consumers correspond to those of PAD synth
+ */
+//currently no enum, since this won't work with rPreset
+//enum consumer_location_t
+//{
+
+#define ad_global_amp 0
+#define ad_global_freq 1
+#define ad_global_filter 2
+
+#define ad_voice_amp 3
+#define ad_voice_freq 4
+#define ad_voice_filter 5
+
+#define ad_voice_fm_amp 6
+#define ad_voice_fm_freq 7
+
+#define sub_freq 8
+#define sub_filter 9
+#define sub_bandwidth 10
+
+#define in_effect 11
+#define loc_unspecified 12
+
+//};
+using consumer_location_t = int;
+
+enum class consumer_location_type_t
+{
+    freq, amp, filter, unspecified
+};
+
 }
 
 #endif
