@@ -18,6 +18,8 @@
 
 #include "Presets.h"
 
+namespace zyn {
+
 /**PresetsArray and Clipboard management*/
 class PresetsArray:public Presets
 {
@@ -39,4 +41,6 @@ class PresetsArray:public Presets
         //virtual void defaults(int n) = 0;
 };
 
+}
+
 #endif
