@@ -28,9 +28,9 @@ struct zest_handles {
     void (*zest_close)(zest_t*);
     void (*zest_setup)(zest_t*);
     void (*zest_draw)(zest_t*);
-    void (*zest_motion)(zest_t*, int x, int y);
-    void (*zest_scroll)(zest_t*, int x, int y, int dx, int dy);
-    void (*zest_mouse)(zest_t *z, int button, int action, int x, int y);
+    void (*zest_motion)(zest_t*, int x, int y, int mod);
+    void (*zest_scroll)(zest_t*, int x, int y, int dx, int dy, int mod);
+    void (*zest_mouse)(zest_t *z, int button, int action, int x, int y, int mod);
     void (*zest_key)(zest_t *z, char *key, bool press);
     void (*zest_resize)(zest_t *z, int w, int h);
     void (*zest_special)(zest_t *z, int key, int press);
@@ -60,6 +60,10 @@ public:
             handle = LoadLibrary("./libzest.dll");
         if(!handle)
             handle = LoadLibrary("libzest.dll");
+#elif defined __APPLE__
+        handle = dlopen("@loader_path/libzest.dylib", RTLD_NOW | RTLD_LOCAL);
+        if(!handle) // VST
+            handle = dlopen("@loader_path/../Resources/libzest.dylib", RTLD_LAZY);
 #else
         handle = dlopen("./libzest.so", RTLD_LAZY);
         if(!handle)
@@ -154,7 +158,7 @@ protected:
     bool onScroll(const ScrollEvent &ev) override
     {
         if(z.zest)
-            z.zest_scroll(z.zest, ev.pos.getX(), ev.pos.getY(), ev.delta.getX(), ev.delta.getY());
+            z.zest_scroll(z.zest, ev.pos.getX(), ev.pos.getY(), ev.delta.getX(), ev.delta.getY(), ev.mod);
         return false;
     }
 
@@ -169,14 +173,14 @@ protected:
     bool onMouse(const MouseEvent &m) override
     {
         if(z.zest)
-            z.zest_mouse(z.zest, m.button, m.press, m.pos.getX(), m.pos.getY());
+            z.zest_mouse(z.zest, m.button, m.press, m.pos.getX(), m.pos.getY(), m.mod);
         return false;
     }
 
     bool onMotion(const MotionEvent &m) override
     {
         if(z.zest)
-            z.zest_motion(z.zest, m.pos.getX(), m.pos.getY());
+            z.zest_motion(z.zest, m.pos.getX(), m.pos.getY(), m.mod);
         return false;
     }
 
@@ -190,8 +194,8 @@ protected:
         if(!z.zest) {
             if(!z.zest_open)
                 return;
-if(!oscPort)
-    return;
+            if(!oscPort)
+                return;
             printf("[INFO:Zyn] zest_open()\n");
             char address[1024];
             snprintf(address, sizeof(address), "osc.udp://127.0.0.1:%d",oscPort);
@@ -203,7 +207,6 @@ if(!oscPort)
         }
 
         z.zest_draw(z.zest);
-        repaint();
     }
 
     bool onKeyboard(const KeyboardEvent &ev)
@@ -218,8 +221,11 @@ if(!oscPort)
 
     void uiIdle(void) override
     {
-        if(z.zest)
-            z.zest_tick(z.zest);
+        if(z.zest) {
+            if (z.zest_tick(z.zest)) {
+                repaint();
+            }
+        }
     }
 
     void uiReshape(uint width, uint height)
