@@ -23,6 +23,7 @@
 #include "../Misc/Allocator.h"
 #include "../Params/ADnoteParameters.h"
 #include "../Containers/ScratchString.h"
+#include "../Containers/NotePool.h"
 #include "ModFilter.h"
 #include "OscilGen.h"
 #include "ADnote.h"
@@ -41,10 +42,12 @@ ADnote::ADnote(ADnoteParameters *pars_, SynthParams &spars,
 
     ADnoteParameters &pars = *pars_;
     portamento  = spars.portamento;
-    midinote    = spars.note;
+    note_log2_freq = spars.note_log2_freq;
     NoteEnabled = ON;
     basefreq    = spars.frequency;
     velocity    = spars.velocity;
+    initial_seed = spars.seed;
+    current_prng_state = spars.seed;
     stereo = pars.GlobalPar.PStereo;
 
     NoteGlobalPar.Detune = getdetune(pars.GlobalPar.PDetuneType,
@@ -53,8 +56,8 @@ ADnote::ADnote(ADnoteParameters *pars_, SynthParams &spars,
     bandwidthDetuneMultiplier = pars.getBandwidthDetuneMultiplier();
 
     if(pars.GlobalPar.PPanning == 0)
-        NoteGlobalPar.Panning = RND;
-    else
+        NoteGlobalPar.Panning = getRandomFloat();
+     else
         NoteGlobalPar.Panning = pars.GlobalPar.PPanning / 128.0f;
 
 
@@ -216,7 +219,7 @@ void ADnote::setupVoice(int nvoice)
     voice.filterbypass = param.Pfilterbypass;
 
     setupVoiceMod(nvoice);
-    
+
     voice.FMVoice = param.PFMVoice;
     voice.FMFreqEnvelope = NULL;
     voice.FMAmpEnvelope  = NULL;
@@ -422,7 +425,7 @@ void ADnote::setupVoiceMod(int nvoice, bool first_run)
     else
         switch(param.PFMEnabled) {
             case 1:
-                voice.FMEnabled = MORPH;
+                voice.FMEnabled = MIX;
                 break;
             case 2:
                 voice.FMEnabled = RING_MOD;
@@ -453,7 +456,7 @@ void ADnote::setupVoiceMod(int nvoice, bool first_run)
 
         float tmp = 1.0f;
         if((pars.VoicePar[vc].FMSmp->Padaptiveharmonics != 0)
-                || (voice.FMEnabled == MORPH)
+                || (voice.FMEnabled == MIX)
                 || (voice.FMEnabled == RING_MOD))
             tmp = getFMvoicebasefreq(nvoice);
 
@@ -482,7 +485,7 @@ void ADnote::setupVoiceMod(int nvoice, bool first_run)
     //Compute the Voice's modulator volume (incl. damping)
     float fmvoldamp = powf(440.0f / getvoicebasefreq(nvoice),
             param.PFMVolumeDamp / 64.0f - 1.0f);
-    const float fmvolume_ = param.PFMVolume / 127.0f;
+    const float fmvolume_ = param.FMvolume / 100.0f;
     switch(voice.FMEnabled) {
         case PHASE_MOD:
         case PW_MOD:
@@ -508,8 +511,9 @@ void ADnote::setupVoiceMod(int nvoice, bool first_run)
 
 SynthNote *ADnote::cloneLegato(void)
 {
-    SynthParams sp{memory, ctl, synth, time, legato.param.freq, velocity, 
-                   (bool)portamento, legato.param.midinote, true};
+    SynthParams sp{memory, ctl, synth, time, legato.param.freq, velocity,
+                (bool)portamento, legato.param.note_log2_freq, true,
+                initial_seed };
     return memory.alloc<ADnote>(&pars, sp);
 }
 
@@ -520,14 +524,15 @@ SynthNote *ADnote::cloneLegato(void)
 void ADnote::legatonote(LegatoParams lpars)
 {
     //ADnoteParameters &pars = *partparams;
-
     // Manage legato stuff
     if(legato.update(lpars))
         return;
 
     portamento = lpars.portamento;
-    midinote   = lpars.midinote;
+    note_log2_freq = lpars.note_log2_freq;
     basefreq   = lpars.frequency;
+    initial_seed = lpars.seed;
+    current_prng_state = lpars.seed;
 
     if(velocity > 1.0f)
         velocity = 1.0f;
@@ -538,12 +543,12 @@ void ADnote::legatonote(LegatoParams lpars)
                                      pars.GlobalPar.PDetune);
     bandwidthDetuneMultiplier = pars.getBandwidthDetuneMultiplier();
 
-    if(pars.GlobalPar.PPanning == 0)
-        NoteGlobalPar.Panning = RND;
-    else
+    if(pars.GlobalPar.PPanning == 0) {
+        NoteGlobalPar.Panning = getRandomFloat();
+    } else
         NoteGlobalPar.Panning = pars.GlobalPar.PPanning / 128.0f;
 
-    NoteGlobalPar.Filter->updateSense(velocity, 
+    NoteGlobalPar.Filter->updateSense(velocity,
                 pars.GlobalPar.PFilterVelocityScale,
                 pars.GlobalPar.PFilterVelocityScaleFunction);
 
@@ -587,13 +592,12 @@ void ADnote::legatonote(LegatoParams lpars)
                 pars.VoicePar[nvoice].PFMCoarseDetune,
                 pars.VoicePar[nvoice].PFMDetune);
 
-
         //Get the voice's oscil or external's voice oscil
         int vc = nvoice;
         if(pars.VoicePar[nvoice].Pextoscil != -1)
             vc = pars.VoicePar[nvoice].Pextoscil;
         if(!pars.GlobalPar.Hrandgrouping)
-            pars.VoicePar[vc].OscilSmp->newrandseed(prng());
+            pars.VoicePar[vc].OscilSmp->newrandseed(getRandomUint());
 
         pars.VoicePar[vc].OscilSmp->get(NoteVoicePar[nvoice].OscilSmp,
                                          getvoicebasefreq(nvoice),
@@ -631,20 +635,20 @@ void ADnote::legatonote(LegatoParams lpars)
                              nvoice), pars.VoicePar[nvoice].PFMVolumeDamp
                          / 64.0f);
                 NoteVoicePar[nvoice].FMVolume =
-                    (expf(pars.VoicePar[nvoice].PFMVolume / 127.0f
+                    (expf(pars.VoicePar[nvoice].FMvolume / 100.0f
                           * FM_AMP_MULTIPLIER) - 1.0f) * fmvoldamp * 4.0f;
                 break;
             case FREQ_MOD:
                 NoteVoicePar[nvoice].FMVolume =
-                    (expf(pars.VoicePar[nvoice].PFMVolume / 127.0f
+                    (expf(pars.VoicePar[nvoice].FMvolume / 100.0f
                           * FM_AMP_MULTIPLIER) - 1.0f) * fmvoldamp * 4.0f;
                 break;
             default:
                 if(fmvoldamp > 1.0f)
                     fmvoldamp = 1.0f;
                 NoteVoicePar[nvoice].FMVolume =
-                    pars.VoicePar[nvoice].PFMVolume
-                    / 127.0f * fmvoldamp;
+                    pars.VoicePar[nvoice].FMvolume
+                    / 100.0f * fmvoldamp;
         }
 
         //Voice's modulator velocity sensing
@@ -657,7 +661,6 @@ void ADnote::legatonote(LegatoParams lpars)
                         * logf(50.0f))
                    - 1.0f) / synth.buffersize_f / 10.0f * synth.samplerate_f);
     }
-
     ///    initparameters();
 
     ///////////////
@@ -666,17 +669,11 @@ void ADnote::legatonote(LegatoParams lpars)
     int tmp[NUM_VOICES];
 
     NoteGlobalPar.Volume = 4.0f
-                           * powf(0.1f, 3.0f
-                                  * (1.0f - pars.GlobalPar.PVolume
-                                     / 96.0f))                                      //-60 dB .. 0 dB
+                           * powf(10.0f, pars.GlobalPar.Volume / 20.0) //-60 dB .. 20 dB
                            * VelF(
         velocity,
         pars.GlobalPar.PAmpVelocityScaleFunction); //velocity sensing
 
-    globalnewamplitude = NoteGlobalPar.Volume
-                         * NoteGlobalPar.AmpEnvelope->envout_dB()
-                         * NoteGlobalPar.AmpLfo->amplfoout();
-
     {
         auto        *filter  = NoteGlobalPar.Filter;
         filter->updateSense(velocity, pars.GlobalPar.PFilterVelocityScale,
@@ -697,19 +694,18 @@ void ADnote::legatonote(LegatoParams lpars)
         NoteVoicePar[nvoice].noisetype = pars.VoicePar[nvoice].Type;
         /* Voice Amplitude Parameters Init */
         NoteVoicePar[nvoice].Volume =
-            powf(0.1f, 3.0f
-                 * (1.0f - pars.VoicePar[nvoice].PVolume / 127.0f))             // -60 dB .. 0 dB
+            dB2rap(pars.VoicePar[nvoice].volume)             // -60 dB .. 0 dB
             * VelF(velocity,
                    pars.VoicePar[nvoice].PAmpVelocityScaleFunction); //velocity
-        if(pars.VoicePar[nvoice].PVolume == 0)
+        if(pars.VoicePar[nvoice].volume == -60.0)
             NoteVoicePar[nvoice].Volume = 0;
 
         if(pars.VoicePar[nvoice].PVolumeminus != 0)
             NoteVoicePar[nvoice].Volume = -NoteVoicePar[nvoice].Volume;
 
-        if(pars.VoicePar[nvoice].PPanning == 0)
-            NoteVoicePar[nvoice].Panning = RND;  // random panning
-        else
+        if(pars.VoicePar[nvoice].PPanning == 0) {
+            NoteVoicePar[nvoice].Panning = getRandomFloat();
+        } else
             NoteVoicePar[nvoice].Panning =
                 pars.VoicePar[nvoice].PPanning / 128.0f;
 
@@ -734,7 +730,7 @@ void ADnote::legatonote(LegatoParams lpars)
            && (NoteVoicePar[nvoice].FMVoice < 0)) {
             pars.VoicePar[nvoice].FMSmp->newrandseed(prng());
 
-            //Perform Anti-aliasing only on MORPH or RING MODULATION
+            //Perform Anti-aliasing only on MIX or RING MODULATION
 
             int vc = nvoice;
             if(pars.VoicePar[nvoice].PextFMoscil != -1)
@@ -858,17 +854,17 @@ void ADnote::initparameters(WatchManager *wm, const char *prefix)
 
         vce.noisetype = param.Type;
         /* Voice Amplitude Parameters Init */
-        vce.Volume = powf(0.1f, 3.0f * (1.0f - param.PVolume / 127.0f)) // -60dB..0dB
+        vce.Volume = dB2rap(param.volume) // -60dB..0dB
                      * VelF(velocity, param.PAmpVelocityScaleFunction);
-        if(param.PVolume == 0)
+        if(param.volume == -60.0f)
             vce.Volume = 0;
 
         if(param.PVolumeminus)
             vce.Volume = -vce.Volume;
 
-        if(param.PPanning == 0)
-            vce.Panning = RND;  // random panning
-        else
+        if(param.PPanning == 0) {
+            vce.Panning = getRandomFloat();
+        } else
             vce.Panning = param.PPanning / 128.0f;
 
         newamplitude[nvoice] = 1.0f;
@@ -924,7 +920,7 @@ void ADnote::initparameters(WatchManager *wm, const char *prefix)
             param.FMSmp->newrandseed(prng());
             vce.FMSmp = memory.valloc<float>(synth.oscilsize + OSCIL_SMP_EXTRA_SAMPLES);
 
-            //Perform Anti-aliasing only on MORPH or RING MODULATION
+            //Perform Anti-aliasing only on MIX or RING MODULATION
 
             int vc = nvoice;
             if(param.PextFMoscil != -1)
@@ -932,7 +928,7 @@ void ADnote::initparameters(WatchManager *wm, const char *prefix)
 
             float tmp = 1.0f;
             if((pars.VoicePar[vc].FMSmp->Padaptiveharmonics != 0)
-               || (vce.FMEnabled == MORPH)
+               || (vce.FMEnabled == MIX)
                || (vce.FMEnabled == RING_MOD))
                 tmp = getFMvoicebasefreq(nvoice);
 
@@ -1072,9 +1068,7 @@ float ADnote::getvoicebasefreq(int nvoice) const
         float fixedfreq   = 440.0f;
         int   fixedfreqET = NoteVoicePar[nvoice].fixedfreqET;
         if(fixedfreqET != 0) { //if the frequency varies according the keyboard note
-            float tmp =
-                (midinote
-                 - 69.0f) / 12.0f
+            float tmp = (note_log2_freq - (69.0f / 12.0f))
                 * (powf(2.0f, (fixedfreqET - 1) / 63.0f) - 1.0f);
             if(fixedfreqET <= 64)
                 fixedfreq *= powf(2.0f, tmp);
@@ -1305,9 +1299,9 @@ inline void ADnote::ComputeVoiceOscillator_CubicInterpolation(int nvoice){
 };
 */
 /*
- * Computes the Oscillator (Morphing)
+ * Computes the Oscillator (Mixing)
  */
-inline void ADnote::ComputeVoiceOscillatorMorph(int nvoice)
+inline void ADnote::ComputeVoiceOscillatorMix(int nvoice)
 {
     ComputeVoiceOscillator_LinearInterpolation(nvoice);
     if(FMnewamplitude[nvoice] > 1.0f)
@@ -1564,15 +1558,15 @@ inline void ADnote::ComputeVoicePinkNoise(int nvoice)
         float *tw = tmpwave_unison[k];
         float *f = &pinking[nvoice][k > 0 ? 7 : 0];
         for(int i = 0; i < synth.buffersize; ++i) {
-	    float white = (RND-0.5)/4.0;
-	    f[0] = 0.99886*f[0]+white*0.0555179;
-	    f[1] = 0.99332*f[1]+white*0.0750759;
-	    f[2] = 0.96900*f[2]+white*0.1538520;
-	    f[3] = 0.86650*f[3]+white*0.3104856;
-	    f[4] = 0.55000*f[4]+white*0.5329522;
-	    f[5] = -0.7616*f[5]-white*0.0168980;
-	    tw[i] = f[0]+f[1]+f[2]+f[3]+f[4]+f[5]+f[6]+white*0.5362;
-	    f[6] = white*0.115926;
+            float white = (RND-0.5)/4.0;
+            f[0] = 0.99886*f[0]+white*0.0555179;
+            f[1] = 0.99332*f[1]+white*0.0750759;
+            f[2] = 0.96900*f[2]+white*0.1538520;
+            f[3] = 0.86650*f[3]+white*0.3104856;
+            f[4] = 0.55000*f[4]+white*0.5329522;
+            f[5] = -0.7616*f[5]-white*0.0168980;
+            tw[i] = f[0]+f[1]+f[2]+f[3]+f[4]+f[5]+f[6]+white*0.5362;
+            f[6] = white*0.115926;
         }
     }
 }
@@ -1621,8 +1615,8 @@ int ADnote::noteout(float *outl, float *outr)
         switch (NoteVoicePar[nvoice].noisetype) {
             case 0: //voice mode=sound
                 switch(NoteVoicePar[nvoice].FMEnabled) {
-                    case MORPH:
-                        ComputeVoiceOscillatorMorph(nvoice);
+                    case MIX:
+                        ComputeVoiceOscillatorMix(nvoice);
                         break;
                     case RING_MOD:
                         ComputeVoiceOscillatorRingModulation(nvoice);
@@ -1980,10 +1974,10 @@ void ADnote::Global::initparameters(const ADnoteGlobalParam &param,
     AmpLfo      = memory.alloc<LFO>(*param.AmpLfo, basefreq, time, wm,
                    (pre+"GlobalPar/AmpLfo/").c_str);
 
-    Volume = 4.0f * powf(0.1f, 3.0f * (1.0f - param.PVolume / 96.0f)) //-60 dB .. 0 dB
+    Volume = 4.0f * powf(10.0f, param.Volume / 20.0) //-60 dB .. 20 dB
              * VelF(velocity, param.PAmpVelocityScaleFunction);     //sensing
 
-    Filter = memory.alloc<ModFilter>(*param.GlobalFilter, synth, time, memory, 
+    Filter = memory.alloc<ModFilter>(*param.GlobalFilter, synth, time, memory,
             stereo, basefreq);
 
     FilterEnvelope = memory.alloc<Envelope>(*param.FilterEnvelope, basefreq,
@@ -1993,7 +1987,7 @@ void ADnote::Global::initparameters(const ADnoteGlobalParam &param,
 
     Filter->addMod(*FilterEnvelope);
     Filter->addMod(*FilterLfo);
-    
+
     {
         Filter->updateSense(velocity, param.PFilterVelocityScale,
                 param.PFilterVelocityScaleFunction);
