@@ -21,6 +21,7 @@
 #include "Envelope.h"
 #include "ModFilter.h"
 #include "../Containers/ScratchString.h"
+#include "../Containers/NotePool.h"
 #include "../Params/Controller.h"
 #include "../Params/SUBnoteParameters.h"
 #include "../Params/FilterParams.h"
@@ -34,7 +35,8 @@
 
 namespace zyn {
 
-SUBnote::SUBnote(const SUBnoteParameters *parameters, SynthParams &spars)
+SUBnote::SUBnote(const SUBnoteParameters *parameters, SynthParams &spars, WatchManager *wm, const char *prefix
+                 )
     :SynthNote(spars), pars(*parameters),
     AmpEnvelope(nullptr),
     FreqEnvelope(nullptr),
@@ -42,10 +44,9 @@ SUBnote::SUBnote(const SUBnoteParameters *parameters, SynthParams &spars)
     GlobalFilter(nullptr),
     GlobalFilterEnvelope(nullptr),
     NoteEnabled(true),
-    lfilter(nullptr), rfilter(nullptr),
-    wm(nullptr)
+    lfilter(nullptr), rfilter(nullptr)
 {
-    setup(spars.frequency, spars.velocity, spars.portamento, spars.note);
+    setup(spars.frequency, spars.velocity, spars.portamento, spars.note_log2_freq, false, wm, prefix);
 }
 
 float SUBnote::setupFilters(int *pos, bool automation)
@@ -90,14 +91,16 @@ float SUBnote::setupFilters(int *pos, bool automation)
 void SUBnote::setup(float freq,
                     float velocity,
                     int portamento_,
-                    int midinote,
-                    bool legato)
+                    float note_log2_freq,
+                    bool legato,
+                    WatchManager *wm,
+                    const char *prefix)
 {
     this->velocity = velocity;
     portamento  = portamento_;
     NoteEnabled = ON;
-    volume      = powf(0.1f, 3.0f * (1.0f - pars.PVolume / 96.0f)); //-60 dB .. 0 dB
-    volume     *= VelF(velocity, pars.PAmpVelocityScaleFunction);
+    volume      = powf(10.0,  pars.Volume / 20.0f);
+    volume     *= VelF(velocity, pars.AmpVelocityScaleFunction);
     if(pars.PPanning != 0)
         panning = pars.PPanning / 127.0f;
     else
@@ -117,7 +120,7 @@ void SUBnote::setup(float freq,
         basefreq = 440.0f;
         int fixedfreqET = pars.PfixedfreqET;
         if(fixedfreqET) { //if the frequency varies according the keyboard note
-            float tmp = (midinote - 69.0f) / 12.0f
+            float tmp = (note_log2_freq - (69.0f / 12.0f))
                 * (powf(2.0f, (fixedfreqET - 1) / 63.0f) - 1.0f);
             if(fixedfreqET <= 64)
                 basefreq *= powf(2.0f, tmp);
@@ -166,18 +169,17 @@ void SUBnote::setup(float freq,
     }
 
     //how much the amplitude is normalised (because the harmonics)
-    float reduceamp = setupFilters(pos, false);
+    float reduceamp = setupFilters(pos, legato);
     oldreduceamp    = reduceamp;
-
     volume /= reduceamp;
 
     oldpitchwheel = 0;
     oldbandwidth  = 64;
     if(!legato) { //normal note
         if(pars.Pfixedfreq == 0)
-            initparameters(basefreq, wm);
+            initparameters(basefreq, wm, prefix);
         else
-            initparameters(basefreq / 440.0f * freq, wm);
+            initparameters(basefreq / 440.0f * freq, wm, prefix);
     }
     else {
         if(pars.Pfixedfreq == 0)
@@ -195,7 +197,7 @@ void SUBnote::setup(float freq,
 SynthNote *SUBnote::cloneLegato(void)
 {
     SynthParams sp{memory, ctl, synth, time, legato.param.freq, velocity,
-                   portamento, legato.param.midinote, true};
+                   portamento, legato.param.note_log2_freq, true, legato.param.seed};
     return memory.alloc<SUBnote>(&pars, sp);
 }
 
@@ -206,8 +208,8 @@ void SUBnote::legatonote(LegatoParams pars)
         return;
 
     try {
-        setup(pars.frequency, pars.velocity, pars.portamento, pars.midinote,
-              true);
+        setup(pars.frequency, pars.velocity, pars.portamento, pars.note_log2_freq,
+              true, wm);
     } catch (std::bad_alloc &ba) {
         std::cerr << "failed to set legato note parameter in SUBnote: " << ba.what() << std::endl;
     }
@@ -353,10 +355,9 @@ void SUBnote::filter(bpfilter &filter, float *smps)
 /*
  * Init Parameters
  */
-void SUBnote::initparameters(float freq, WatchManager *wm)
+void SUBnote::initparameters(float freq, WatchManager *wm, const char *prefix)
 {
-    //TODO populate this base string
-    ScratchString pre;
+    ScratchString pre = prefix;
     AmpEnvelope = memory.alloc<Envelope>(*pars.AmpEnvelope, freq,
             synth.dt(), wm, (pre+"AmpEnvelope/").c_str);
 
