@@ -10,6 +10,7 @@
   of the License, or (at your option) any later version.
 */
 #include "SynthNote.h"
+#include "../Misc/Util.h"
 #include "../globals.h"
 #include <cstring>
 #include <new>
@@ -20,11 +21,11 @@ namespace zyn {
 SynthNote::SynthNote(SynthParams &pars)
     :memory(pars.memory),
     legato(pars.synth, pars.frequency, pars.velocity, pars.portamento,
-            pars.note, pars.quiet), ctl(pars.ctl), synth(pars.synth), time(pars.time)
+            pars.note_log2_freq, pars.quiet, pars.seed), ctl(pars.ctl), synth(pars.synth), time(pars.time)
 {}
 
 SynthNote::Legato::Legato(const SYNTH_T &synth_, float freq, float vel, int port,
-                          int note, bool quiet)
+                          float note_log2_freq, bool quiet, prng_t seed)
     :synth(synth_)
 {
     // Initialise some legato-specific vars
@@ -37,7 +38,8 @@ SynthNote::Legato::Legato(const SYNTH_T &synth_, float freq, float vel, int port
     param.freq = freq;
     param.vel  = vel;
     param.portamento = port;
-    param.midinote   = note;
+    param.note_log2_freq = note_log2_freq;
+    param.seed = seed;
     lastfreq = 0.0f;
     silent   = quiet;
 }
@@ -51,7 +53,7 @@ int SynthNote::Legato::update(LegatoParams pars)
         param.freq = pars.frequency;
         param.vel  = pars.velocity;
         param.portamento = pars.portamento;
-        param.midinote   = pars.midinote;
+        param.note_log2_freq = pars.note_log2_freq;
         if(msg == LM_Norm) {
             if(silent) {
                 fade.m = 0.0f;
@@ -90,7 +92,7 @@ void SynthNote::Legato::apply(SynthNote &note, float *outl, float *outr)
                         decounter = -10;
                         msg = LM_ToNorm;
                         LegatoParams pars{param.freq, param.vel, param.portamento,
-                                          param.midinote, false};
+                                          param.note_log2_freq, false, param.seed};
                         note.legatonote(pars);
                         break;
                     }
@@ -132,7 +134,7 @@ void SynthNote::Legato::apply(SynthNote &note, float *outl, float *outr)
                         //previous freq during the fadeout.
                         float catchupfreq = param.freq * (param.freq / lastfreq);
                         LegatoParams pars{catchupfreq, param.vel, param.portamento,
-                                          param.midinote, false};
+                                          param.note_log2_freq, false, param.seed};
                         note.legatonote(pars);
                         break;
                     }
@@ -152,7 +154,7 @@ void SynthNote::Legato::apply(SynthNote &note, float *outl, float *outr)
 void SynthNote::setVelocity(float velocity_) {
     legato.setSilent(true); //Let legato.update(...) returns 0.
     LegatoParams pars{legato.getFreq(), velocity_,
-               legato.getPortamento(), legato.getMidinote(), true};
+               legato.getPortamento(), legato.getNoteLog2Freq(), true, legato.getSeed()};
     try {
         legatonote(pars);
     } catch (std::bad_alloc &ba) {
@@ -161,4 +163,13 @@ void SynthNote::setVelocity(float velocity_) {
     legato.setDecounter(0); //avoid chopping sound due fade-in
 }
 
+float SynthNote::getRandomFloat() {
+    return (getRandomUint() / (INT32_MAX * 1.0f));
+}
+
+prng_t SynthNote::getRandomUint() {
+    current_prng_state = prng_r(current_prng_state);
+    return current_prng_state;
+}
+
 }
