@@ -20,6 +20,7 @@
 */
 
 #include "WatchPoint.h"
+#include "../Misc/Util.h"
 #include <cstring>
 #include <rtosc/thread-link.h>
 
@@ -30,9 +31,9 @@ WatchPoint::WatchPoint(WatchManager *ref, const char *prefix, const char *id)
 {
     identity[0] = 0;
     if(prefix)
-        strncpy(identity, prefix, 128);
+        fast_strcpy(identity, prefix, sizeof(identity));
     if(id)
-        strncat(identity, id, 128);
+        strncat(identity, id, sizeof(identity));
 }
 
 bool WatchPoint::is_active(void)
@@ -50,7 +51,7 @@ bool WatchPoint::is_active(void)
 
     return false;
 }
-    
+
 FloatWatchPoint::FloatWatchPoint(WatchManager *ref, const char *prefix, const char *id)
     :WatchPoint(ref, prefix, id)
 {}
@@ -58,7 +59,7 @@ FloatWatchPoint::FloatWatchPoint(WatchManager *ref, const char *prefix, const ch
 VecWatchPoint::VecWatchPoint(WatchManager *ref, const char *prefix, const char *id)
     :WatchPoint(ref, prefix, id)
 {}
-    
+
 WatchManager::WatchManager(thrlnk *link)
     :write_back(link), new_active(false)
 {
@@ -67,7 +68,7 @@ WatchManager::WatchManager(thrlnk *link)
     memset(data_list,   0, sizeof(data_list));
     memset(deactivate,  0, sizeof(deactivate));
 }
-    
+
 void WatchManager::add_watch(const char *id)
 {
     //Don't add duplicate watchs
@@ -78,7 +79,7 @@ void WatchManager::add_watch(const char *id)
     //Apply to a free slot
     for(int i=0; i<MAX_WATCH; ++i) {
         if(!active_list[i][0]) {
-            strncpy(active_list[i], id, 128);
+            fast_strcpy(active_list[i], id, MAX_WATCH_PATH);
             new_active = true;
             sample_list[i] = 0;
             break;
@@ -136,7 +137,7 @@ bool WatchManager::active(const char *id) const
 
     return false;
 }
-    
+
 int WatchManager::samples(const char *id) const
 {
     for(int i=0; i<MAX_WATCH; ++i)
@@ -144,7 +145,7 @@ int WatchManager::samples(const char *id) const
             return sample_list[i];
     return 0;
 }
-    
+
 void WatchManager::satisfy(const char *id, float f)
 {
     //printf("trying to satisfy '%s'\n", id);
