@@ -91,7 +91,7 @@ class AdNoteTest:public CxxTest::TestSuite
             //lets go with.... 50! as a nice note
             testnote = 50;
             float freq = 440.0f * powf(2.0f, (testnote - 69.0f) / 12.0f);
-            SynthParams pars{memory, *controller, *synth, *time, freq, 120, 0, testnote, false};
+            SynthParams pars{memory, *controller, *synth, *time, freq, 120, 0, testnote / 12.0f, false, prng()};
 
             note = new ADnote(defaultPreset, pars);
 
@@ -124,26 +124,26 @@ class AdNoteTest:public CxxTest::TestSuite
 #endif
             sampleCount += synth->buffersize;
 
-            TS_ASSERT_DELTA(outL[255], 0.254609f, 0.0001f);
+            TS_ASSERT_DELTA(outL[255], 0.2555f, 0.0001f);
 
             note->releasekey();
 
 
             note->noteout(outL, outR);
             sampleCount += synth->buffersize;
-            TS_ASSERT_DELTA(outL[255], -0.102197f, 0.0001f);
+            TS_ASSERT_DELTA(outL[255], -0.4688f, 0.0001f);
 
             note->noteout(outL, outR);
             sampleCount += synth->buffersize;
-            TS_ASSERT_DELTA(outL[255], -0.111261f, 0.0001f);
+            TS_ASSERT_DELTA(outL[255], 0.0613f, 0.0001f);
 
             note->noteout(outL, outR);
             sampleCount += synth->buffersize;
-            TS_ASSERT_DELTA(outL[255], -0.021375f, 0.0001f);
+            TS_ASSERT_DELTA(outL[255], 0.0971f, 0.0001f);
 
             note->noteout(outL, outR);
             sampleCount += synth->buffersize;
-            TS_ASSERT_DELTA(outL[255], 0.149149f, 0.0001f);
+            TS_ASSERT_DELTA(outL[255], -0.0901f, 0.0001f);
 
             while(!note->finished()) {
                 note->noteout(outL, outR);
