@@ -23,6 +23,7 @@
 #include "../Synth/ADnote.h"
 #include "../Synth/OscilGen.h"
 #include "../Params/Presets.h"
+#include "../Params/FilterParams.h"
 #include "../DSP/FFTwrapper.h"
 #include "../globals.h"
 using namespace std;
@@ -64,6 +65,9 @@ class AdNoteTest:public CxxTest::TestSuite
             //sawtooth to make things a bit more interesting
             params->VoicePar[0].OscilSmp->Pcurrentbasefunc = 3;
 
+            params->GlobalPar.PFilterVelocityScale = 64;
+            params->GlobalPar.GlobalFilter->basefreq = 5076.203125;
+
             controller = new Controller(*synth, time);
 
             //lets go with.... 50! as a nice note
@@ -84,7 +88,7 @@ class AdNoteTest:public CxxTest::TestSuite
         void run_test(int a, int b, int c, int d, int e, int f, float values[4])
         {
             sprng(0);
-            
+
             const int ADnote_unison_sizes[] =
             {1, 2, 3, 4, 5, 6, 8, 10, 12, 15, 20, 25, 30, 40, 50, 0};
             params->VoicePar[0].Unison_size = ADnote_unison_sizes[a];
@@ -94,20 +98,20 @@ class AdNoteTest:public CxxTest::TestSuite
             params->VoicePar[0].Unison_vibratto_speed   = e;
             params->VoicePar[0].Unison_invert_phase     = f;
 
-            SynthParams pars{memory, *controller, *synth, *time, freq, 120, 0, testnote, false};
+            SynthParams pars{memory, *controller, *synth, *time, freq, 120, 0, testnote / 12.0f, false, prng()};
             note = new ADnote(params, pars);
             note->noteout(outL, outR);
             TS_ASSERT_DELTA(outL[80], values[0], 1e-5);
-            //printf("{%f,", outL[80]);
+            printf("\n{%f,", outL[80]);
             note->noteout(outL, outR);
             TS_ASSERT_DELTA(outR[90], values[1], 1e-5);
-            //printf("%f,", outR[90]);
+            printf("\n%f,", outR[90]);
             note->noteout(outL, outR);
             TS_ASSERT_DELTA(outL[20], values[2], 1e-5);
-            //printf("%f,", outL[20]);
+            printf("\n%f,", outL[20]);
             note->noteout(outL, outR);
             TS_ASSERT_DELTA(outR[200], values[3], 1e-5);
-            //printf("%f},\n", outR[200]);
+            printf("\n%f},\n", outR[200]);
         }
 
         void testUnison() {
@@ -115,20 +119,20 @@ class AdNoteTest:public CxxTest::TestSuite
 
             float data[][4] = {
                 {-0.034547,0.034349,-0.000000,0.138284},
-                {0.023612,-0.093842,0.000000,-0.040384},
-                {-0.015980,0.001871,-0.014463,-0.000726},
-                {-0.040970,-0.000275,0.000000,-0.121016},
-                {0.019250,-0.045252,0.000270,0.105372},
-                {-0.086575,0.001130,-0.018921,0.001329},
-                {0.009203,-0.006176,0.017344,-0.003316},
-                {0.029411,-0.000248,-0.112797,-0.012883},
-                {0.043657,-0.014062,-0.003374,-0.071821},
-                {0.007973,0.068019,-0.038900,0.047639},
-                {-0.002055,0.011170,-0.058152,-0.043493},
-                {-0.005298,0.000605,-0.070932,-0.005678},
-                {0.025028,-0.027742,0.020985,-0.015417},
-                {0.074349,0.000640,0.080613,0.066636},
-                {-0.045721,0.000279,0.009819,0.032202},
+                {0.016801,-0.084991,0.000000,0.009240},
+                {0.020383,-0.002424,-0.012952,-0.014037},
+                {-0.041653,0.002287,0.000000,-0.098181},
+                {-0.009189,-0.049860,0.000268,-0.084961},
+                {0.056999,-0.084627,-0.018144,0.000666},
+                {-0.015588,0.003690,0.003994,0.002435},
+                {0.023178,-0.024961,0.004433,-0.015144},
+                {0.042007,-0.006559,-0.005887,0.083685},
+                {0.007638,0.057870,-0.014244,0.041457},
+                {-0.018006,-0.017846,-0.063624,-0.016378},
+                {0.004914,-0.001756,-0.046715,0.015975},
+                {0.004341,-0.014575,0.000560,0.050902},
+                {0.000470,-0.036961,0.038622,0.031383},
+                {-0.045796,0.000262,0.009858,0.031958},
             };
 
             int freq_spread[15];
